"""
* 文件名: test_smcp_namespace
* 作者: JQQ
* 创建日期: 2025/9/29
* 最后修改日期: 2025/9/29
* 版权: 2023 JQQ. All rights reserved.
* 依赖: pytest, socketio
* 描述: SMCP Namespace测试用例 / SMCP Namespace test cases
"""

from unittest.mock import AsyncMock, MagicMock

import pytest

from a2c_smcp.server import (
    AuthenticationProvider,
    DefaultAuthenticationProvider,
    SMCPNamespace,
)
from a2c_smcp.smcp import (
    GET_DESKTOP_EVENT,
    SMCP_NAMESPACE,
    UPDATE_DESKTOP_EVENT,
    UPDATE_DESKTOP_NOTIFICATION,
    EnterOfficeReq,
    GetDeskTopReq,
    GetDeskTopRet,
    LeaveOfficeReq,
)


class MockAuthProvider(AuthenticationProvider):
    """Mock认证提供者用于测试 / Mock authentication provider for testing"""

    async def authenticate(self, sio: AsyncMock, environ: dict, auth: dict | None, headers: list) -> bool:
        """简单的Mock认证逻辑 / Simple mock authentication logic"""
        # 从headers中提取API密钥进行认证
        # Extract API key from headers for authentication
        for header in headers:
            if isinstance(header, (list, tuple)) and len(header) >= 2:
                header_name = header[0].decode("utf-8").lower() if isinstance(header[0], bytes) else str(header[0]).lower()
                header_value = header[1].decode("utf-8") if isinstance(header[1], bytes) else str(header[1])

                if header_name == "x-api-key" and header_value == "valid_key":
                    return True
        return False

    async def has_admin_permission(self, sio: AsyncMock, agent_id: str, secret: str) -> bool:
        """Mock管理员权限检查 / Mock admin permission check"""
        return secret == "admin_secret"


@pytest.fixture
def mock_auth_provider():
    """创建Mock认证提供者 / Create mock authentication provider"""
    return MockAuthProvider()


@pytest.fixture
def smcp_namespace(mock_auth_provider):
    """创建SMCP命名空间实例 / Create SMCP namespace instance"""
    return SMCPNamespace(mock_auth_provider)


@pytest.fixture
def mock_server():
    """创建Mock服务器 / Create mock server"""
    server = AsyncMock()
    server.app = MagicMock()
    server.app.state = MagicMock()
    server.app.state.agent_id = "test_agent"
    server.manager = MagicMock()
    return server


class TestSMCPNamespace:
    """SMCP命名空间测试类 / SMCP namespace test class"""

    @pytest.mark.asyncio
    async def test_namespace_initialization(self, smcp_namespace):
        """测试命名空间初始化 / Test namespace initialization"""
        assert smcp_namespace.namespace == SMCP_NAMESPACE
        assert isinstance(smcp_namespace.auth_provider, MockAuthProvider)

    @pytest.mark.asyncio
    async def test_successful_connection(self, smcp_namespace, mock_server):
        """测试成功连接 / Test successful connection"""
        smcp_namespace.server = mock_server

        # Mock环境变量和认证数据
        # Mock environment variables and auth data
        environ = {
            "asgi": {
                "scope": {
                    "headers": [
                        (b"x-api-key", b"valid_key"),
                    ],
                },
            },
        }

        # 测试连接
        # Test connection
        result = await smcp_namespace.on_connect("test_sid", environ, None)
        assert result is True

    @pytest.mark.asyncio
    async def test_failed_authentication(self, smcp_namespace, mock_server):
        """测试认证失败 / Test authentication failure"""
        smcp_namespace.server = mock_server

        # Mock环境变量和无效认证数据
        # Mock environment variables and invalid auth data
        environ = {
            "asgi": {
                "scope": {
                    "headers": [
                        (b"x-api-key", b"invalid_key"),
                    ],
                },
            },
        }

        # 测试连接应该失败
        # Test connection should fail
        with pytest.raises(ConnectionRefusedError):
            await smcp_namespace.on_connect("test_sid", environ, None)

    @pytest.mark.asyncio
    async def test_join_office_success(self, smcp_namespace):
        """测试成功加入房间 / Test successful office join"""
        # Mock会话数据
        # Mock session data
        session = {}
        smcp_namespace.get_session = AsyncMock(return_value=session)
        smcp_namespace.save_session = AsyncMock()
        smcp_namespace.enter_room = AsyncMock()

        # 测试数据
        # Test data
        data = EnterOfficeReq(**{
            "role": "computer",
            "name": "test_computer",
            "office_id": "office_123",
        })

        # 执行测试
        # Execute test
        success, error = await smcp_namespace.on_server_join_office("test_sid", data)

        assert success is True
        assert error is None
        assert session["role"] == "computer"
        assert session["name"] == "test_computer"

    @pytest.mark.asyncio
    async def test_join_office_role_mismatch(self, smcp_namespace):
        """测试角色不匹配的情况 / Test role mismatch scenario"""
        # Mock会话数据，已有不同角色
        # Mock session data with different existing role
        session = {"role": "agent"}
        smcp_namespace.get_session = AsyncMock(return_value=session)
        smcp_namespace.save_session = AsyncMock()

        # 测试数据
        # Test data
        data = EnterOfficeReq(**{
            "role": "computer",
            "name": "test_computer",
            "office_id": "office_123",
        })

        # 执行测试
        # Execute test
        success, error = await smcp_namespace.on_server_join_office("test_sid", data)

        assert success is False
        assert "Role mismatch" in error

    @pytest.mark.asyncio
    async def test_leave_office(self, smcp_namespace):
        """测试离开房间 / Test leaving office"""
        smcp_namespace.leave_room = AsyncMock()

        # 测试数据
        # Test data
        data = LeaveOfficeReq(**{"office_id": "office_123"})

        # 执行测试
        # Execute test
        success, error = await smcp_namespace.on_server_leave_office("test_sid", data)

        assert success is True
        assert error is None
        smcp_namespace.leave_room.assert_called_once_with("test_sid", "office_123")

    @pytest.mark.asyncio
    async def test_enter_room_agent_constraints_and_duplicate(self, smcp_namespace, mock_server):
        """覆盖 agent 进入房间的约束：
        - 已在其他房间 -> 抛错
        - 已在同一房间 -> 警告并返回（不重复加入）
        - 房间内已存在 agent -> 抛错
        """
        smcp_namespace.server = mock_server

        # 1) agent 已在其他房间
        session = {"role": "agent", "office_id": "roomA"}
        smcp_namespace.get_session = AsyncMock(return_value=session)
        smcp_namespace.save_session = AsyncMock()
        smcp_namespace.emit = AsyncMock()

        with pytest.raises(ValueError):
            await smcp_namespace.enter_room("sid1", "roomB")

        # 2) agent 未在任何房间，但房间内已有 agent
        session2 = {"role": "agent"}
        smcp_namespace.get_session = AsyncMock(return_value=session2)
        # 房间已有一个 agent 参与者
        mock_server.manager.get_participants.return_value = ["sidAgent"]

        smcp_namespace.get_session = AsyncMock(side_effect=[session2, {"role": "agent"}])
        with pytest.raises(ValueError):
            await smcp_namespace.enter_room("sid2", "roomA")

        # 3) agent 已在同一房间 -> 返回（不抛错）
        session3 = {"role": "agent", "office_id": "roomA"}
        smcp_namespace.get_session = AsyncMock(return_value=session3)
        # 调用不会抛出
        await smcp_namespace.enter_room("sid3", "roomA")

    @pytest.mark.asyncio
    async def test_enter_room_computer_switch_and_duplicate(self, smcp_namespace, mock_server):
        """覆盖 computer 切换房间与重复加入同一房间。"""
        smcp_namespace.server = mock_server
        # computer 从 roomA 切到 roomB，应先 leave_room(roomA)
        session = {"role": "computer", "office_id": "roomA"}
        smcp_namespace.get_session = AsyncMock(return_value=session)
        smcp_namespace.save_session = AsyncMock()
        smcp_namespace.emit = AsyncMock()
        smcp_namespace.leave_room = AsyncMock()
        await smcp_namespace.enter_room("csid", "roomB")
        smcp_namespace.leave_room.assert_called_once_with("csid", "roomA")

        # 重复加入同一房间 -> 直接返回
        session2 = {"role": "computer", "office_id": "roomB"}
        smcp_namespace.get_session = AsyncMock(return_value=session2)
        await smcp_namespace.enter_room("csid", "roomB")

    @pytest.mark.asyncio
    async def test_leave_room_broadcast_and_clear_session(self, smcp_namespace, monkeypatch):
        """覆盖 leave_room 的广播通知与 session 清理。"""
        smcp_namespace.emit = AsyncMock()
        sess = {"role": "computer", "office_id": "roomX"}
        smcp_namespace.get_session = AsyncMock(return_value=sess)
        smcp_namespace.save_session = AsyncMock()
        # 拦截 BaseNamespace.leave_room，避免真实父类逻辑
        from a2c_smcp.server.base import BaseNamespace

        monkeypatch.setattr(BaseNamespace, "leave_room", AsyncMock())
        await smcp_namespace.leave_room("sidX", "roomX")
        assert "office_id" not in sess

    @pytest.mark.asyncio
    async def test_on_client_get_tools_and_tool_call_and_updates(self, smcp_namespace, mock_server):
        """覆盖 get_tools 权限校验、tool_call 与 update/cancel 的广播/调用。"""
        smcp_namespace.server = mock_server

        # 准备会话：agent 与 computer 在同一房间
        agent_id = "a1"
        agent_sid = "a_sid1"
        comp_name = "c1"
        comp_sid = "c_sid1"
        sess_agent = {"role": "agent", "office_id": "room1", "name": agent_id}
        sess_comp = {"role": "computer", "office_id": "room1", "name": comp_name}

        smcp_namespace.get_session = AsyncMock(side_effect=lambda sid: (sess_comp if sid == comp_sid else sess_agent))
        smcp_namespace._name_to_sid_map = {comp_name: comp_sid, agent_id: agent_sid}
        smcp_namespace.call = AsyncMock(return_value={"tools": [], "req_id": "r1"})

        # get_tools 成功
        ret = await smcp_namespace.on_client_get_tools(agent_id, {"computer": comp_name})
        assert isinstance(ret, dict)
        assert "tools" in ret

        # tool_call：由 agent 发起，映射到 call
        smcp_namespace.get_session = AsyncMock(return_value=sess_agent)
        smcp_namespace.call = AsyncMock(return_value={"ok": True})
        res = await smcp_namespace.on_client_tool_call(
            agent_id,
            {
                "agent": agent_id,
                "req_id": "r2",
                "computer": comp_name,
                "tool_name": "t1",
                "params": {},
                "timeout": 5,
            },
        )
        assert res == {"ok": True}

        # update_config：由 computer 发起，广播通知
        smcp_namespace.get_session = AsyncMock(return_value=sess_comp)
        smcp_namespace.emit = AsyncMock()
        await smcp_namespace.on_server_update_config(comp_name, {"computer": comp_name})
        smcp_namespace.emit.assert_awaited()

        # cancel tool call：由 agent 发起，广播通知
        smcp_namespace.get_session = AsyncMock(return_value=sess_agent)
        smcp_namespace.emit = AsyncMock()
        await smcp_namespace.on_server_tool_call_cancel(agent_id, {"agent": agent_id, "req_id": "r3"})
        smcp_namespace.emit.assert_awaited()

    @pytest.mark.asyncio
    async def test_on_server_list_room_success(self, smcp_namespace, mock_server, monkeypatch):
        """
        测试成功列出房间内所有会话信息
        Test successfully listing all sessions in a room
        """
        smcp_namespace.server = mock_server

        # 准备测试数据：Agent 和两个 Computer 在同一房间
        # Prepare test data: Agent and two Computers in the same room
        agent_sid = "agent_1"
        comp_sid_1 = "comp_1"
        comp_sid_2 = "comp_2"
        office_id = "test_office"

        # Mock 会话数据 / Mock session data
        sessions_data = [
            {"sid": agent_sid, "name": "Test Agent", "role": "agent", "office_id": office_id},
            {"sid": comp_sid_1, "name": "Computer 1", "role": "computer", "office_id": office_id},
            {"sid": comp_sid_2, "name": "Computer 2", "role": "computer", "office_id": office_id},
        ]

        # Mock get_session 返回 Agent 会话 / Mock get_session to return Agent session
        smcp_namespace.get_session = AsyncMock(
            return_value={"sid": agent_sid, "name": "Test Agent", "role": "agent", "office_id": office_id},
        )

        # 使用 monkeypatch Mock aget_all_sessions_in_office
        # Use monkeypatch to mock aget_all_sessions_in_office
        async def mock_aget_all_sessions(office_id_param, sio):
            return sessions_data

        monkeypatch.setattr("a2c_smcp.server.namespace.aget_all_sessions_in_office", mock_aget_all_sessions)

        # 执行测试 / Execute test
        result = await smcp_namespace.on_server_list_room(
            agent_sid,
            {"agent": agent_sid, "req_id": "req_123", "office_id": office_id},
        )

        # 验证结果 / Verify result
        assert result["req_id"] == "req_123"
        assert len(result["sessions"]) == 3
        assert all(s["office_id"] == office_id for s in result["sessions"])
        assert any(s["sid"] == agent_sid for s in result["sessions"])
        assert any(s["sid"] == comp_sid_1 for s in result["sessions"])
        assert any(s["sid"] == comp_sid_2 for s in result["sessions"])

    @pytest.mark.asyncio
    async def test_on_server_list_room_permission_denied(self, smcp_namespace, mock_server):
        """
        测试权限检查：Agent 只能查询自己所在房间
        Test permission check: Agent can only query their own room
        """
        from a2c_smcp.smcp import ListRoomReq

        smcp_namespace.server = mock_server

        # Agent 在 office_A，但请求查询 office_B
        # Agent is in office_A but requests to query office_B
        agent_sid = "agent_1"
        agent_session = {"sid": agent_sid, "name": "Test Agent", "role": "agent", "office_id": "office_A"}

        smcp_namespace.get_session = AsyncMock(return_value=agent_session)

        # 执行测试，应该抛出 AssertionError / Execute test, should raise AssertionError
        with pytest.raises(AssertionError, match="Agent只能查询自己所在房间的会话信息"):
            await smcp_namespace.on_server_list_room(
                agent_sid,
                {"agent": agent_sid, "req_id": "req_456", "office_id": "office_B"},
            )

    @pytest.mark.asyncio
    async def test_on_server_list_room_filters_invalid_sessions(self, smcp_namespace, mock_server, monkeypatch):
        """
        测试过滤无效会话：只返回有效的 computer 和 agent 角色
        Test filtering invalid sessions: only return valid computer and agent roles
        """
        smcp_namespace.server = mock_server

        agent_sid = "agent_1"
        office_id = "test_office"

        # Mock 会话数据，包含一些无效的会话 / Mock session data with some invalid sessions
        sessions_data = [
            {"sid": agent_sid, "name": "Test Agent", "role": "agent", "office_id": office_id},
            {"sid": "comp_1", "name": "Computer 1", "role": "computer", "office_id": office_id},
            {"sid": "invalid_1", "name": "Invalid", "role": "unknown", "office_id": office_id},  # 无效角色
            {"sid": "comp_2", "name": "Computer 2", "role": "computer", "office_id": office_id},
        ]

        agent_session = {"sid": agent_sid, "name": "Test Agent", "role": "agent", "office_id": office_id}
        smcp_namespace.get_session = AsyncMock(return_value=agent_session)

        # 使用 monkeypatch Mock aget_all_sessions_in_office
        async def mock_aget_all_sessions(office_id_param, sio):
            return sessions_data

        monkeypatch.setattr("a2c_smcp.server.namespace.aget_all_sessions_in_office", mock_aget_all_sessions)

        result = await smcp_namespace.on_server_list_room(
            agent_sid,
            {"agent": agent_sid, "req_id": "req_789", "office_id": office_id},
        )

        # 验证结果：应该只包含 3 个有效会话（排除 unknown 角色）
        # Verify result: should only contain 3 valid sessions (excluding unknown role)
        assert len(result["sessions"]) == 3
        assert all(s["role"] in ["computer", "agent"] for s in result["sessions"])


class TestDefaultAuthenticationProvider:
    """默认认证提供者测试类 / Default authentication provider test class"""

    def test_initialization(self):
        """测试初始化 / Test initialization"""
        provider = DefaultAuthenticationProvider("admin_secret", "custom-api-key")
        assert provider.admin_secret == "admin_secret"
        assert provider.api_key_name == "custom-api-key"

    @pytest.mark.asyncio
    async def test_admin_authentication(self):
        """测试管理员认证 / Test admin authentication"""
        provider = DefaultAuthenticationProvider("admin_secret")
        mock_sio = AsyncMock()

        headers = [(b"x-api-key", b"admin_secret")]
        result = await provider.authenticate(mock_sio, "agent_123", None, headers)
        assert result is True

    @pytest.mark.asyncio
    async def test_failed_authentication(self):
        """测试认证失败 / Test authentication failure"""
        provider = DefaultAuthenticationProvider("admin_secret")
        mock_sio = AsyncMock()

        headers = [(b"x-api-key", b"wrong_secret")]
        result = await provider.authenticate(mock_sio, "agent_123", None, headers)
        assert result is False

    @pytest.mark.asyncio
    async def test_no_api_key(self):
        """测试无API密钥的情况 / Test no API key scenario"""
        provider = DefaultAuthenticationProvider("admin_secret")
        mock_sio = AsyncMock()

        headers = []  # 空的headers列表
        result = await provider.authenticate(mock_sio, "agent_123", None, headers)
        assert result is False



# -*- coding: utf-8 -*-
# filename: test_manager.py
# @Time    : 2025/8/18 14:59
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
import asyncio
from typing import Any, cast
from unittest.mock import AsyncMock, MagicMock

import pytest
from mcp import StdioServerParameters, Tool
from mcp.client.session_group import SseServerParameters, StreamableHttpParameters
from mcp.types import CallToolResult

from a2c_smcp.computer.mcp_clients.manager import MCPServerManager, ToolNameDuplicatedError
from a2c_smcp.computer.mcp_clients.model import (
    MCPClientProtocol,
    MCPServerConfig,
    SseServerConfig,
    StdioServerConfig,
    StreamableHttpServerConfig,
    ToolMeta,
)

# 模拟类型定义
TOOL_NAME = str
SERVER_NAME = str


# 模拟BaseMCPClient
class MockMCPClient:
    def __init__(self, tools: list[Tool] = None, ret_meta: dict | None = None, message_handler=None):
        self.tools = tools or []
        self.aconnect = AsyncMock()
        self.adisconnect = AsyncMock()
        self.list_tools = AsyncMock(return_value=tools)
        call_ret = MagicMock(spec=CallToolResult)
        call_ret.result = None
        call_ret.meta = ret_meta
        self.call_tool = AsyncMock(return_value=call_ret)
        self.state = "connected"
        # 保存透传进来的 message_handler，便于测试断言
        self.message_handler = message_handler


def create_mock_tool(name: str, meta: dict | None = None) -> Tool:
    tool = MagicMock(name=name, spec=Tool)
    tool.name = name
    tool.meta = meta
    return tool


# 模拟client_factory函数
def mock_client_factory(config: MCPServerConfig, message_handler=None) -> MockMCPClient:
    # 简化处理：根据配置名称返回不同的工具列表
    if "server1" in config.name:
        return MockMCPClient([create_mock_tool("tool1", meta={"test": "meta"}), create_mock_tool("tool2")], message_handler=message_handler)
    elif "server2" in config.name:
        return MockMCPClient(
            [create_mock_tool("tool3"), create_mock_tool("tool4")],
            ret_meta={"test": "ret_meta"},
            message_handler=message_handler,
        )
    elif "alias_server" in config.name:
        return MockMCPClient([create_mock_tool("tool5")], message_handler=message_handler)
    elif "duplicate_server" in config.name:
        return MockMCPClient([create_mock_tool("duplicate_tool")], message_handler=message_handler)
    return MockMCPClient(message_handler=message_handler)


# Monkey patch客户端工厂函数
@pytest.fixture(autouse=True)
def patch_client_factory(monkeypatch):
    monkeypatch.setattr("a2c_smcp.computer.mcp_clients.manager.client_factory", mock_client_factory)


# 创建示例服务器配置
def create_server_config(
    name: str,
    disabled: bool = False,
    forbidden_tools: list = None,
    tool_meta: dict = None,
    default_tool_meta: ToolMeta | None = None,
) -> MCPServerConfig:
    forbidden_tools = forbidden_tools or []
    tool_meta = tool_meta or {}
    if "sse" in name:
        return SseServerConfig(
            name=name,
            disabled=disabled,
            forbidden_tools=forbidden_tools,
            tool_meta=tool_meta,
            default_tool_meta=default_tool_meta,
            server_parameters=MagicMock(spec=SseServerParameters),
        )
    elif "http" in name:
        return StreamableHttpServerConfig(
            name=name,
            disabled=disabled,
            forbidden_tools=forbidden_tools,
            tool_meta=tool_meta,
            default_tool_meta=default_tool_meta,
            server_parameters=MagicMock(spec=StreamableHttpParameters),
        )
    else:
        return StdioServerConfig(
            name=name,
            disabled=disabled,
            forbidden_tools=forbidden_tools,
            tool_meta=tool_meta,
            default_tool_meta=default_tool_meta,
            server_parameters=MagicMock(spec=StdioServerParameters),
        )


@pytest.fixture
async def manager() -> MCPServerManager:
    manager = MCPServerManager()
    await manager.enable_auto_reconnect()  # 启用自动重连便于测试
    return manager


@pytest.mark.asyncio
async def test_initialize_with_servers(manager):
    """测试初始化和服务器启动"""
    servers = [create_server_config("server1"), create_server_config("server2", disabled=True), create_server_config("sse_server")]

    await manager.ainitialize(servers)

    # 初始化后不会自动启动所有服务。验证活动客户端
    assert "server1" not in manager._active_clients
    assert "sse_server" not in manager._active_clients
    assert "server2" not in manager._active_clients

    # 调用start_all
    await manager.astart_all()

    # 验证启动
    assert "server1" in manager._active_clients
    assert "sse_server" in manager._active_clients
    assert "server2" not in manager._active_clients

    # 验证工具映射
    assert manager._tool_mapping["tool1"] == "server1"
    assert manager._tool_mapping["tool2"] == "server1"
    assert "tool3" not in manager._tool_mapping  # 禁用的服务器

    # 验证状态检查
    statuses = manager.get_server_status()
    assert ("server1", True, "connected") in statuses
    assert ("server2", False, "pending") in statuses
    assert ("sse_server", True, "connected") in statuses


@pytest.mark.asyncio
async def test_tool_execution(manager):
    """测试工具执行流程"""
    servers = [create_server_config("server1")]
    await manager.ainitialize(servers)

    await manager.astart_all()

    # 执行工具
    params = {"key": "value"}
    await manager.aexecute_tool("tool1", params)

    # 验证调用
    client = manager._active_clients["server1"]
    client.call_tool.assert_awaited_once_with("tool1", params)

    with pytest.raises(Exception):
        await manager.aexecute_tool("tool5", params)


@pytest.mark.asyncio
async def test_tool_execution_with_ret_meta(manager):
    """测试工具执行流程"""
    servers = [create_server_config("server2", tool_meta={"tool3": ToolMeta(ret_meta={"test": "ret_meta"})})]
    await manager.ainitialize(servers)

    await manager.astart_all()

    # 执行工具
    params = {"key": "value"}
    ret = await manager.aexecute_tool("tool3", params)
    assert ret.meta["test"] == "ret_meta"

    # 验证调用
    client = manager._active_clients["server2"]
    client.call_tool.assert_awaited_once_with("tool3", params)


@pytest.mark.asyncio
async def test_tool_with_alias(manager):
    """测试别名映射功能"""
    tool_meta = {"tool5": ToolMeta(alias="aliased_tool")}
    servers = [create_server_config("alias_server", tool_meta=tool_meta)]
    await manager.ainitialize(servers)
    await manager.astart_all()

    # 验证别名映射
    assert manager._alias_mapping["aliased_tool"] == ("alias_server", "tool5")
    assert "tool5" not in manager._tool_mapping
    assert manager._tool_mapping["aliased_tool"] == "alias_server"

    # 执行别名工具
    await manager.aexecute_tool("aliased_tool", {})
    client = manager._active_clients["alias_server"]
    print("Call args list:", client.call_tool.call_args_list)
    client.call_tool.assert_awaited_once_with("tool5", {})


@pytest.mark.asyncio
async def test_disabled_tool(manager):
    """测试禁用工具处理"""
    servers = [create_server_config("server1", forbidden_tools=["tool2"])]
    await manager.ainitialize(servers)
    await manager.astart_all()

    # 验证禁用状态
    assert "tool2" in manager._disabled_tools

    # 尝试执行禁用工具
    with pytest.raises(PermissionError):
        await manager.aexecute_tool("tool2", {})


@pytest.mark.asyncio
async def test_tool_name_conflict(manager):
    """测试工具名冲突处理"""
    servers = [
        create_server_config("server1"),
        create_server_config("duplicate_server", tool_meta={"duplicate_tool": ToolMeta(alias="tool1")}),
    ]

    # 验证初始化时检测到冲突
    with pytest.raises(ToolNameDuplicatedError):
        await manager.ainitialize(servers)
        await manager.astart_all()

    # 工具重名导致的异常是在逐个启动Client的时候抛出的，因此只会回滚检测到异常的Client，
    # 而不会回滚所有Client
    assert len(manager._active_clients) == 1


@pytest.mark.asyncio
async def test_dynamic_server_management(manager):
    """测试动态添加/移除服务器"""
    # 初始配置
    servers = [create_server_config("server1")]
    await manager.ainitialize(servers)
    await manager.astart_all()
    assert "server1" in manager._active_clients

    # 添加新服务器
    new_server = create_server_config("http_server")
    await manager.aadd_or_aupdate_server(new_server)

    # 验证新服务器启动
    assert "http_server" not in manager._active_clients
    await manager._astart_client("http_server")
    assert "http_server" in manager._active_clients

    # 更新服务器配置（启用自动重连）
    updated_server = create_server_config("server1", forbidden_tools=["tool1"])
    # 验证服务器重启
    old_client = manager._active_clients["server1"]  # 要提示保存旧客户端的引用，因为add_or_update_server会销毁旧客户端
    await manager.aadd_or_aupdate_server(updated_server)
    await asyncio.sleep(0.1)  # 等待自动重连 需要释放一次协程才能触发协程任务的执行与调用。

    old_client.adisconnect.assert_awaited()

    # 验证更新应用
    assert "tool1" in manager._disabled_tools

    # 移除服务器
    await manager.aremove_server("http_server")
    assert "http_server" not in manager._active_clients
    assert "http_server" not in manager._servers_config


@pytest.mark.asyncio
async def test_auto_reconnect_disabled(manager):
    """测试禁用自动重连时更新配置"""
    await manager.disable_auto_reconnect()

    servers = [create_server_config("server1")]
    await manager.ainitialize(servers)
    await manager.astart_all()

    # 尝试更新活动服务器的配置
    updated_config = create_server_config("server1", forbidden_tools=["tool1"])

    with pytest.raises(RuntimeError):
        await manager.aadd_or_aupdate_server(updated_config)


@pytest.mark.asyncio
async def test_get_available_tools(manager):
    """测试获取可用工具"""
    tool_meta = {"tool1": ToolMeta(auto_apply=True)}
    servers = [create_server_config("server1", tool_meta=tool_meta)]
    await manager.ainitialize(servers)
    await manager.astart_all()

    # 获取工具
    tools = []
    async for tool in manager.available_tools():
        tools.append(tool)

    assert len(tools) == 2
    tool1 = next(t for t in tools if t.name == "tool1")
    assert tool1.meta["a2c_tool_meta"].auto_apply


@pytest.mark.asyncio
async def test_default_tool_meta_applies_when_missing_per_tool(manager):
    """当未提供 per-tool 配置时，应回落使用 default_tool_meta。
    When per-tool meta is missing, default_tool_meta should be applied.
    """
    servers = [create_server_config("server1", default_tool_meta=ToolMeta(auto_apply=True))]
    await manager.ainitialize(servers)
    await manager.astart_all()

    # 检查 available_tools 注入
    tools = []
    async for tool in manager.available_tools():
        tools.append(tool)
    t1 = next(t for t in tools if t.name == "tool1")
    assert t1.meta["a2c_tool_meta"].auto_apply is True

    # 检查 aexecute_tool 返回元数据注入
    ret = await manager.aexecute_tool("tool1", {})
    assert ret.meta["a2c_tool_meta"].auto_apply is True


@pytest.mark.asyncio
async def test_per_tool_overrides_default(manager):
    """per-tool 配置应覆盖 default_tool_meta 的根级字段。
    Per-tool meta should override default_tool_meta root-level fields.
    """
    servers = [
        create_server_config(
            "server1",
            tool_meta={"tool1": ToolMeta(auto_apply=False)},
            default_tool_meta=ToolMeta(auto_apply=True),
        ),
    ]
    await manager.ainitialize(servers)
    await manager.astart_all()

    ret = await manager.aexecute_tool("tool1", {})
    assert ret.meta["a2c_tool_meta"].auto_apply is False


@pytest.mark.asyncio
async def test_error_handling(manager):
    """测试错误处理"""
    # 模拟客户端连接错误
    bad_server = create_server_config("error_server")
    bad_client = MockMCPClient()
    bad_client.list_tools.side_effect = Exception("Connection failed")
    manager.client_factory = lambda _: bad_client

    await manager.aadd_or_aupdate_server(bad_server)

    # 验证状态
    assert "error_server" not in manager._active_clients

    # 工具执行错误处理
    servers = [create_server_config("server1")]
    await manager.ainitialize(servers)
    await manager.astart_all()

    client = manager._active_clients["server1"]
    client.call_tool.side_effect = TimeoutError("Execution timed out")

    with pytest.raises(TimeoutError):
        await manager.aexecute_tool("tool1", {}, timeout=0.1)


@pytest.mark.asyncio
async def test_meta_data_injection(manager):
    """测试工具元数据注入"""
    tool_meta = {"tool1": ToolMeta(ret_object_mapper={"result": "data"})}
    servers = [create_server_config("server1", tool_meta=tool_meta)]
    await manager.ainitialize(servers)
    await manager.astart_all()

    # 执行工具
    result = await manager.aexecute_tool("tool1", {})

    # 验证元数据注入
    assert "a2c_tool_meta" in result.meta
    assert result.meta["a2c_tool_meta"].ret_object_mapper == {"result": "data"}


@pytest.mark.asyncio
async def test_manager_propagates_message_handler_to_clients():
    """验证 Manager 能将 message_handler 透传到具体 Client。
    Verify Manager forwards message_handler to concrete clients.
    """

    # 定义一个占位回调
    async def dummy_handler(*args, **kwargs):
        return None

    mgr = MCPServerManager(message_handler=dummy_handler)
    await mgr.enable_auto_reconnect()

    servers = [create_server_config("server1"), create_server_config("sse_server")]
    await mgr.ainitialize(servers)
    await mgr.astart_all()

    # 校验每个激活客户端都收到了相同的回调实例
    for name, client in mgr._active_clients.items():
        assert getattr(client, "message_handler", None) is dummy_handler, f"Client {name} did not receive message_handler"


@pytest.mark.asyncio
async def test_manager_message_handler_none_results_in_none_on_clients():
    """当未提供 message_handler 时，客户端应为 None。"""
    mgr = MCPServerManager()  # 不传入 handler
    await mgr.enable_auto_reconnect()

    servers = [create_server_config("server1")]
    await mgr.ainitialize(servers)
    await mgr.astart_all()

    client = mgr._active_clients["server1"]
    assert getattr(client, "message_handler", None) is None


# 覆盖 _add_server_config 的 RuntimeError 分支
# Test _add_server_config RuntimeError branch
@pytest.mark.asyncio
async def test_update_active_server_without_reconnect(manager):
    """
    测试：当 auto_reconnect=False 且尝试更新已激活的服务器配置时抛出 RuntimeError。
    Test: Raise RuntimeError when updating active config with auto_reconnect=False.
    """
    config = create_server_config("server1")
    await manager.disable_auto_reconnect()
    manager._servers_config[config.name] = config
    manager._active_clients[config.name] = cast(MCPClientProtocol, MagicMock(spec=MCPClientProtocol))
    with pytest.raises(RuntimeError):
        await manager._add_or_update_server_config(config)


# 覆盖 astart_client 的 ValueError/RuntimeError 分支
# Test astart_client ValueError/RuntimeError branches
@pytest.mark.asyncio
async def test_astart_client_invalid_cases(manager):
    """
    测试：启动未知服务器/禁用服务器时报错。
    Test: Raise ValueError/RuntimeError for unknown or disabled server.
    """
    # 未知服务器
    with pytest.raises(ValueError):
        await manager._astart_client("not_exist")
    # 禁用服务器
    config = create_server_config("server2", disabled=True)
    manager._servers_config[config.name] = config
    with pytest.raises(RuntimeError):
        await manager._astart_client(config.name)


# 覆盖 aexecute_tool 的 PermissionError/ValueError/RuntimeError 分支
# Test aexecute_tool PermissionError/ValueError/RuntimeError branches
@pytest.mark.asyncio
async def test_aexecute_tool_invalid_cases(manager):
    """
    测试：执行被禁用工具/未注册工具/服务器未激活时报错。
    Test: Raise PermissionError/ValueError/RuntimeError for disabled tool, missing tool, or inactive server.
    """
    # 工具被禁用
    manager._disabled_tools.add("toolX")
    with pytest.raises(PermissionError):
        await manager.aexecute_tool("toolX", {})
    # 工具未注册
    with pytest.raises(ValueError):
        await manager.aexecute_tool("no_tool", {})
    # 工具所在服务器未激活
    manager._tool_mapping["toolY"] = "serverY"
    manager._active_clients.clear()
    manager._servers_config["serverY"] = create_server_config("serverY")
    with pytest.raises(RuntimeError):
        await manager.aexecute_tool("toolY", {})


# 覆盖 aexecute_tool 的 TimeoutError/Exception 分支
# Test aexecute_tool TimeoutError/Exception branches
@pytest.mark.asyncio
async def test_aexecute_tool_timeout_and_exception(manager, monkeypatch):
    """
    测试：执行工具时超时或抛出异常。
    Test: Raise TimeoutError/RuntimeError when tool execution times out or raises.
    """
    config = create_server_config("server3")
    manager._servers_config[config.name] = config
    manager._active_clients[config.name] = cast(MCPClientProtocol, MagicMock(spec=MCPClientProtocol))
    manager._tool_mapping["toolZ"] = config.name
    mock_client = manager._active_clients[config.name]
    # 超时
    mock_client.call_tool = AsyncMock(side_effect=asyncio.TimeoutError)
    with pytest.raises(TimeoutError):
        await manager.aexecute_tool("toolZ", {}, timeout=0.01)
    # 其它异常
    mock_client.call_tool = AsyncMock(side_effect=Exception("fail"))
    with pytest.raises(RuntimeError):
        await manager.aexecute_tool("toolZ", {})


# 覆盖 _arefresh_tool_mapping 的 ToolNameDuplicatedError 分支
# Test _arefresh_tool_mapping ToolNameDuplicatedError branch
@pytest.mark.asyncio
async def test_arefresh_tool_mapping_duplicate(manager, monkeypatch):
    """
    测试：多个服务器存在同名工具时抛出 ToolNameDuplicatedError。
    Test: Raise ToolNameDuplicatedError when duplicate tool name exists across servers.
    """
    # 两个 server 都返回同名工具 duplicate_tool
    config1 = create_server_config("duplicate_server1")
    config2 = create_server_config("duplicate_server2")
    # 强制都只返回同名工具 duplicate_tool

    def always_duplicate_tool(_, message_handler=None):
        return MockMCPClient([create_mock_tool("duplicate_tool")], message_handler=message_handler)

    monkeypatch.setattr("a2c_smcp.computer.mcp_clients.manager.client_factory", always_duplicate_tool)
    manager._servers_config = {config1.name: config1, config2.name: config2}
    manager._active_clients = {config1.name: always_duplicate_tool(config1), config2.name: always_duplicate_tool(config2)}
    with pytest.raises(ToolNameDuplicatedError):
        await manager._arefresh_tool_mapping()


@pytest.mark.asyncio
async def test_astart_client(manager, monkeypatch):
    """
    测试：多个服务器存在同名工具时抛出 ToolNameDuplicatedError。
    Test: Raise ToolNameDuplicatedError when duplicate tool name exists across servers.
    """
    # 两个 server 都返回同名工具 duplicate_tool
    config1 = create_server_config("duplicate_server1")
    config2 = create_server_config("duplicate_server2")
    # 强制都只返回同名工具 duplicate_tool

    def always_duplicate_tool(_, message_handler=None):
        return MockMCPClient([create_mock_tool("duplicate_tool")], message_handler=message_handler)

    monkeypatch.setattr("a2c_smcp.computer.mcp_clients.manager.client_factory", always_duplicate_tool)
    await manager.aadd_or_aupdate_server(config1)
    await manager.aadd_or_aupdate_server(config2)
    assert not manager._active_clients
    await manager.astart_client(config1.name)
    assert manager._active_clients
    assert len(manager._active_clients) == 1
    with pytest.raises(ToolNameDuplicatedError):
        await manager.astart_client(config2.name)
    await manager.astop_client(config1.name)
    assert not manager._active_clients
    await manager.astart_client(config2.name)
    assert manager._active_clients
    assert len(manager._active_clients) == 1


# 覆盖 aremove_server 的删除不存在服务器分支
# Test aremove_server deleting non-existent server
@pytest.mark.asyncio
async def test_aremove_server_not_exist(manager):
    """
    测试：移除不存在的服务器配置应抛出 KeyError。
    Test: Raise KeyError when removing a non-existent server config.
    """
    with pytest.raises(KeyError):
        await manager.aremove_server("not_exist")


@pytest.mark.asyncio
async def test_astart_all_tool_name_duplicate(manager, monkeypatch):
    """
    覆盖 astart_all 的工具名重复异常分支。
    Cover the duplicate tool name exception branch in astart_all.
    """
    # 确保 manager 状态干净
    await manager.aclose()
    config1 = create_server_config("dup_server1")
    config2 = create_server_config("dup_server2")

    # 两个 server 都返回同名工具 duplicate_tool
    def always_duplicate_tool(_, message_handler=None):
        return MockMCPClient([create_mock_tool("duplicate_tool")], message_handler=message_handler)

    monkeypatch.setattr("a2c_smcp.computer.mcp_clients.manager.client_factory", always_duplicate_tool)
    servers = [config1, config2]
    # 因为首次初始化时配置未连接，也就不会检查工具名冲突，因此可以正常初始化
    await manager.ainitialize(servers)
    with pytest.raises(ToolNameDuplicatedError):
        await manager.astart_all()


@pytest.mark.asyncio
async def test_aadd_or_aupdate_server_with_duplicate_tool(manager, monkeypatch):
    """
    测试：在添加/更新服务器时遇到工具名重复会抛出ToolNameDuplicatedError
    Test: Raise ToolNameDuplicatedError when adding/updating server with duplicate tool name
    """
    # 初始配置
    await manager.enable_auto_connect()
    config1 = create_server_config("server1")
    await manager.ainitialize([config1])
    await manager.astart_all()

    # 模拟工具名重复的情况
    def duplicate_tool_factory(config: MCPServerConfig, message_handler=None) -> Any:
        if config.name == "server1":
            return MockMCPClient([create_mock_tool("tool1")], message_handler=message_handler)
        else:
            return MockMCPClient([create_mock_tool("tool1")], message_handler=message_handler)  # 故意返回同名工具

    monkeypatch.setattr("a2c_smcp.computer.mcp_clients.manager.client_factory", duplicate_tool_factory)

    # 添加新服务器（会触发工具名冲突）
    config2 = create_server_config("server2")
    with pytest.raises(ToolNameDuplicatedError):
        await manager.aadd_or_aupdate_server(config2)

    # 验证状态回滚
    assert "server2" not in manager._active_clients
    assert "server2" not in manager._servers_config  # 因为异常导致回滚

    # 更新现有服务器（会触发工具名冲突）
    config1_updated = create_server_config("server1", tool_meta={"tool1": ToolMeta(alias="new_alias")})
    await manager.aadd_or_aupdate_server(config1_updated)
    await manager.aadd_or_aupdate_server(config2)

    # 验证服务器仍然保持原状
    assert manager._active_clients["server1"].list_tools.return_value[0].name == "tool1"
    tools_list = [tool async for tool in manager.available_tools()]
    assert any(tool.name == "tool1" and tool.meta["a2c_tool_meta"].alias == "new_alias" for tool in tools_list) and any(
        tool.name == "tool1" and not tool.meta for tool in tools_list
    )


@pytest.mark.asyncio
async def test_acall_tool_with_vrl_context_injection(manager, monkeypatch):
    """
    测试：acall_tool在VRL转换时注入tool_name和parameters
    Test: acall_tool injects tool_name and parameters during VRL transformation
    """
    import json

    from mcp.types import TextContent

    from a2c_smcp.computer.mcp_clients.model import A2C_VRL_TRANSFORMED

    # 中文: 创建带VRL脚本的配置，脚本会提取tool_name和parameters
    # English: Create config with VRL script that extracts tool_name and parameters
    vrl_script = """
    .context = {
        "tool": .tool_name,
        "params": .parameters
    }
    """

    # 中文: 创建mock客户端，返回带内容的结果
    # English: Create mock client that returns result with content
    def vrl_test_factory(config: MCPServerConfig, message_handler=None) -> Any:
        mock_result = CallToolResult(
            content=[TextContent(text="test result", type="text")],
            isError=False,
        )
        client = MockMCPClient([create_mock_tool("test_tool")], message_handler=message_handler)
        client.call_tool = AsyncMock(return_value=mock_result)
        return client

    monkeypatch.setattr("a2c_smcp.computer.mcp_clients.manager.client_factory", vrl_test_factory)

    # 中文: 创建配置并初始化
    # English: Create config and initialize
    config = StdioServerConfig(
        name="vrl_test_server",
        disabled=False,
        forbidden_tools=[],
        tool_meta={},
        vrl=vrl_script,
        server_parameters=MagicMock(spec=StdioServerParameters),
    )

    await manager.enable_auto_connect()
    await manager.ainitialize([config])
    await manager.astart_all()

    # 中文: 调用工具，传入参数
    # English: Call tool with parameters
    test_params = {"query": "test query", "limit": 10}
    result = await manager.acall_tool("vrl_test_server", "test_tool", test_params)

    # 中文: 验证VRL转换结果包含tool_name和parameters
    # English: Verify VRL transformation result contains tool_name and parameters
    assert result.meta is not None
    assert A2C_VRL_TRANSFORMED in result.meta

    transformed = json.loads(result.meta[A2C_VRL_TRANSFORMED])
    assert "context" in transformed
    assert transformed["context"]["tool"] == "test_tool"
    assert transformed["context"]["params"] == test_params


@pytest.mark.asyncio
async def test_acall_tool_vrl_conditional_on_tool_name(manager, monkeypatch):
    """
    测试：VRL脚本基于tool_name执行条件逻辑
    Test: VRL script executes conditional logic based on tool_name
    """
    import json

    from mcp.types import TextContent

    from a2c_smcp.computer.mcp_clients.model import A2C_VRL_TRANSFORMED

    # 中文: VRL脚本根据tool_name设置不同的result_type
    # English: VRL script sets different result_type based on tool_name
    vrl_script = """
    if .tool_name == "search" {
        .result_type = "search_result"
        .query = .parameters.query
    } else if .tool_name == "execute" {
        .result_type = "execution_result"
        .command = .parameters.cmd
    } else {
        .result_type = "unknown"
    }
    """

    def conditional_vrl_factory(config: MCPServerConfig, message_handler=None) -> Any:
        mock_result = CallToolResult(
            content=[TextContent(text="result", type="text")],
            isError=False,
        )
        client = MockMCPClient(
            [create_mock_tool("search"), create_mock_tool("execute"), create_mock_tool("other")],
            message_handler=message_handler,
        )
        client.call_tool = AsyncMock(return_value=mock_result)
        return client

    monkeypatch.setattr("a2c_smcp.computer.mcp_clients.manager.client_factory", conditional_vrl_factory)

    config = StdioServerConfig(
        name="conditional_server",
        disabled=False,
        forbidden_tools=[],
        tool_meta={},
        vrl=vrl_script,
        server_parameters=MagicMock(spec=StdioServerParameters),
    )

    await manager.enable_auto_connect()
    await manager.ainitialize([config])
    await manager.astart_all()

    # 中文: 测试search工具
    # English: Test search tool
    result1 = await manager.acall_tool("conditional_server", "search", {"query": "test"})
    transformed1 = json.loads(result1.meta[A2C_VRL_TRANSFORMED])
    assert transformed1["result_type"] == "search_result"
    assert transformed1["query"] == "test"

    # 中文: 测试execute工具
    # English: Test execute tool
    result2 = await manager.acall_tool("conditional_server", "execute", {"cmd": "ls"})
    transformed2 = json.loads(result2.meta[A2C_VRL_TRANSFORMED])
    assert transformed2["result_type"] == "execution_result"
    assert transformed2["command"] == "ls"

    # 中文: 测试其他工具
    # English: Test other tool
    result3 = await manager.acall_tool("conditional_server", "other", {})
    transformed3 = json.loads(result3.meta[A2C_VRL_TRANSFORMED])
    assert transformed3["result_type"] == "unknown"


@pytest.mark.asyncio
async def test_acall_tool_vrl_with_nested_parameters(manager, monkeypatch):
    """
    测试：VRL脚本访问嵌套的parameters字段
    Test: VRL script accesses nested parameters fields
    """
    import json

    from mcp.types import TextContent

    from a2c_smcp.computer.mcp_clients.model import A2C_VRL_TRANSFORMED

    # 中文: VRL脚本访问嵌套的parameters
    # English: VRL script accesses nested parameters
    vrl_script = """
    .user_info = {
        "id": .parameters.user.id,
        "name": .parameters.user.name
    }
    .options = .parameters.options
    """

    def nested_params_factory(config: MCPServerConfig, message_handler=None) -> Any:
        mock_result = CallToolResult(
            content=[TextContent(text="result", type="text")],
            isError=False,
        )
        client = MockMCPClient([create_mock_tool("nested_tool")], message_handler=message_handler)
        client.call_tool = AsyncMock(return_value=mock_result)
        return client

    monkeypatch.setattr("a2c_smcp.computer.mcp_clients.manager.client_factory", nested_params_factory)

    config = StdioServerConfig(
        name="nested_server",
        disabled=False,
        forbidden_tools=[],
        tool_meta={},
        vrl=vrl_script,
        server_parameters=MagicMock(spec=StdioServerParameters),
    )

    await manager.enable_auto_connect()
    await manager.ainitialize([config])
    await manager.astart_all()

    # 中文: 调用工具，传入嵌套参数
    # English: Call tool with nested parameters
    nested_params = {"user": {"id": 123, "name": "Alice"}, "options": {"enabled": True, "timeout": 30}}

    result = await manager.acall_tool("nested_server", "nested_tool", nested_params)
    transformed = json.loads(result.meta[A2C_VRL_TRANSFORMED])

    assert transformed["user_info"]["id"] == 123
    assert transformed["user_info"]["name"] == "Alice"
    assert transformed["options"]["enabled"] is True
    assert transformed["options"]["timeout"] == 30



# -*- coding: utf-8 -*-
# filename: test_computer.py
# @Time    : 2025/8/26
# @Author  : JQQ
# @Email   : jiaqia@qknode.com
# @Software: PyCharm
"""
集成测试：Computer.aexecute_tool 工具调用
Integration test: Computer.aexecute_tool tool execution
"""

from unittest.mock import MagicMock

import pytest
from prompt_toolkit import PromptSession

from a2c_smcp.computer.computer import Computer
from a2c_smcp.computer.mcp_clients.model import SseServerConfig, StdioServerConfig, ToolMeta


@pytest.mark.anyio
async def test_computer_aexecute_tool_success(stdio_params, sse_params, sse_server) -> None:
    """
    测试通过Computer.aexecute_tool调用工具（自动通过）
    Test Computer.aexecute_tool with auto_apply tool

    StdioServer中只有一个可用工具 hello 参数为name 返回值 f"Hello, {name}!"
    """
    stdio_cfg = StdioServerConfig(name="stdio_server", server_parameters=stdio_params, tool_meta={"hello": ToolMeta(auto_apply=True)})
    sse_cfg = SseServerConfig(name="sse_server", server_parameters=sse_params)
    computer = Computer(name="test", mcp_servers={stdio_cfg, sse_cfg})
    await computer.boot_up()
    # 获取可用工具/Get available tools
    tools = await computer.aget_available_tools()
    assert tools, "No tools available"
    tool_name = "hello"
    # 调用工具/Call tool
    result = await computer.aexecute_tool("reqid", tool_name, {"name": "China"})
    assert hasattr(result, "content")
    assert result.content, "Tool call result should have content"
    assert result.content[0].text == "Hello, China!"


@pytest.mark.anyio
async def test_computer_aexecute_tool_confirm_callback_called(stdio_params):
    """
    测试tool_meta为空时会触发二次确认回调，并保证回调被调用
    Test that when tool_meta is empty, confirm_callback is triggered and called
    """
    from a2c_smcp.computer.mcp_clients.model import StdioServerConfig

    # tool_meta为空/No auto_apply
    confirm_mock = MagicMock(return_value=True)
    stdio_cfg = StdioServerConfig(name="stdio_server", server_parameters=stdio_params, tool_meta={})
    computer = Computer(name="test", mcp_servers={stdio_cfg}, confirm_callback=confirm_mock)
    await computer.boot_up()
    await computer.aget_available_tools()
    tool_name = "hello"
    result = await computer.aexecute_tool("reqid", tool_name, {"name": "China"})
    assert hasattr(result, "content")
    assert confirm_mock.called, "confirm_callback should be called"
    assert result.content[0].text == "Hello, China!"


# -------------------- 以下为动态管理相关集成用例 --------------------


class _DictResolver:
    """简单的 resolver：根据映射返回值，模拟 inputs 解析。"""

    def __init__(self, mapping: dict[str, object]):
        self._m = mapping

    async def aresolve_by_id(self, input_id: str, *, session: PromptSession | None = None):  # noqa: D401
        return self._m[input_id]

    def clear_cache(self, key: str | None = None):  # noqa: D401
        return None


@pytest.mark.anyio
async def test_dynamic_add_with_inputs_and_tool_call(stdio_params) -> None:
    """
    动态添加：使用带有 ${input:...} 的原始 dict 配置，确保渲染后能连接 stdio 测试服务并调用 hello 工具。
    """
    # 将fixture提供的参数拆出值，作为 inputs 渲染的来源
    cmd = stdio_params.command
    args = list(stdio_params.args or [])
    cwd = stdio_params.cwd
    env = stdio_params.env

    # 原始字典配置，包含占位符
    cfg_dict: dict = {
        "type": "stdio",
        "name": "dyn_stdio",
        "disabled": False,
        "server_parameters": {
            "command": "${input:cmd}",
            "args": ["${input:arg0}"] + (["${input:arg1}"] if len(args) > 1 else []),
            "env": env,
            "cwd": cwd,
            "encoding": "utf-8",
            "encoding_error_handler": "strict",
        },
        "forbidden_tools": [],
        "tool_meta": {"hello": ToolMeta(auto_apply=True).model_dump(mode="json")},
    }

    # 提供解析器，返回与 fixture 相符的值
    resolver = _DictResolver(
        {
            "cmd": cmd,
            "arg0": args[0] if args else "",
            "arg1": args[1] if len(args) > 1 else "",
        },
    )

    # 初始化 Computer（auto_connect=true 便于动态添加后立即连接）
    computer = Computer(name="test", auto_connect=True, input_resolver=resolver)

    # 动态添加
    await computer.aadd_or_aupdate_server(cfg_dict)

    # 获取工具并调用
    tools = await computer.aget_available_tools()
    assert any(t["name"] == "hello" for t in tools)

    ret = await computer.aexecute_tool("reqid", "hello", {"name": "China"})
    assert ret.content and ret.content[0].text == "Hello, China!"


@pytest.mark.anyio
async def test_dynamic_update_forbid_tool_then_call_fails(stdio_params) -> None:
    """
    动态更新：先添加服务，后通过 forbidden_tools 禁用 hello 工具，随后调用应失败。
    """
    computer = Computer(name="test", auto_connect=True, confirm_callback=lambda *_: True)
    # 先添加（直接用模型，不涉及 inputs）
    cfg = StdioServerConfig(name="dyn_stdio2", server_parameters=stdio_params)
    await computer.aadd_or_aupdate_server(cfg)

    # 确认可调用
    ret1 = await computer.aexecute_tool("reqid", "hello", {"name": "China"})
    assert ret1.content and ret1.content[0].text == "Hello, China!"

    # 更新：禁用工具
    cfg2 = StdioServerConfig(name="dyn_stdio2", server_parameters=stdio_params, forbidden_tools=["hello"])
    await computer.aadd_or_aupdate_server(cfg2)

    # 调用应被 Manager 拒绝
    with pytest.raises(PermissionError):
        await computer.aexecute_tool("reqid", "hello", {"name": "China"})


@pytest.mark.anyio
async def test_dynamic_remove_then_tool_not_found(stdio_params) -> None:
    """
    动态移除：移除后再调用工具应失败（找不到工具）。
    """
    computer = Computer(name="test", auto_connect=True, confirm_callback=lambda *_: True)
    cfg = StdioServerConfig(name="dyn_stdio3", server_parameters=stdio_params)
    await computer.aadd_or_aupdate_server(cfg)

    # 调用成功一次
    ret1 = await computer.aexecute_tool("reqid", "hello", {"name": "China"})
    assert ret1.content and ret1.content[0].text == "Hello, China!"

    # 移除
    await computer.aremove_server("dyn_stdio3")

    # 再次调用应报错：工具不存在
    with pytest.raises(ValueError):
        await computer.aexecute_tool("reqid", "hello", {"name": "China"})


# -------------------- 新增：工具调用历史集成用例 --------------------


@pytest.mark.anyio
async def test_tool_call_history_records_success_and_order(stdio_params) -> None:
    """
    中文：验证 aexecute_tool 成功调用后，历史中包含记录，且字段正确、顺序正确。
    English: Verify history has records after successful calls with correct fields and order.
    """
    cfg = StdioServerConfig(name="hist_stdio", server_parameters=stdio_params, tool_meta={"hello": ToolMeta(auto_apply=True)})
    computer = Computer(name="test", mcp_servers={cfg})
    await computer.boot_up()

    # 进行两次调用/Two calls
    r1 = await computer.aexecute_tool("req-1", "hello", {"name": "A"})
    r2 = await computer.aexecute_tool("req-2", "hello", {"name": "B"})
    assert r1.content and r2.content

    hist = await computer.aget_tool_call_history()
    assert len(hist) >= 2

    # 校验最后一条为第二次调用/Last is second call
    last = hist[-1]
    assert last["req_id"] == "req-2"
    assert last["tool"] == "hello"
    assert last["server"] == "hist_stdio"
    assert last["parameters"] == {"name": "B"}
    assert last["timeout"] is None
    assert last["success"] is True
    assert isinstance(last["timestamp"], str) and last["timestamp"]


@pytest.mark.anyio
async def test_tool_call_history_maxlen(stdio_params) -> None:
    """
    中文：验证历史仅保留最近10条。
    English: Verify history keeps only last 10 records.
    """
    cfg = StdioServerConfig(name="hist_maxlen", server_parameters=stdio_params, tool_meta={"hello": ToolMeta(auto_apply=True)})
    computer = Computer(name="test", mcp_servers={cfg})
    await computer.boot_up()

    # 执行12次，maxlen=10后应只保留后10次/Execute 12 times, keep last 10
    for i in range(12):
        rid = f"req-{i + 1}"
        await computer.aexecute_tool(rid, "hello", {"name": str(i + 1)})

    hist = await computer.aget_tool_call_history()
    assert len(hist) == 10
    # 保留的是 3..12 的 req_id，其中最后一条是 12
    assert hist[0]["req_id"] == "req-3"
    assert hist[-1]["req_id"] == "req-12"


@pytest.mark.anyio
async def test_tool_call_history_confirm_callback_exception(stdio_params) -> None:
    """
    中文：当确认回调抛出异常时，应记录一条失败的历史记录。
    English: When confirm callback raises, a failed history record should be stored.
    """

    def bad_confirm(*_args, **_kwargs):
        raise RuntimeError("confirm failed")

    cfg = StdioServerConfig(name="hist_confirm_err", server_parameters=stdio_params, tool_meta={})
    computer = Computer(name="test", mcp_servers={cfg}, confirm_callback=bad_confirm)
    await computer.boot_up()

    result = await computer.aexecute_tool("req-X", "hello", {"name": "Err"})
    assert result.isError is True

    hist = await computer.aget_tool_call_history()
    assert len(hist) >= 1
    last = hist[-1]
    assert last["req_id"] == "req-X"
    assert last["tool"] == "hello"
    assert last["server"] == "hist_confirm_err"
    assert last["success"] is False
    assert last["error"] is not None



"""
文件名: utils.py
作者: JQQ
创建日期: 2025/9/25
最后修改日期: 2025/9/25
版权: 2023 JQQ. All rights reserved.
依赖: rich
描述:
  中文: CLI 层通用工具：键值解析与表格打印，统一 Console 管理。
  English: Common CLI utilities: key-value parsing and table printers with unified Console management.
"""

from __future__ import annotations

import importlib
import json
from typing import Any

from rich.table import Table

from a2c_smcp.computer.computer import Computer
from a2c_smcp.computer.utils import console as console_util
from a2c_smcp.smcp import SMCPTool

# 使用全局 Console（引用模块属性，便于后续动态切换）
# Use a global Console (module attribute reference for dynamic switching)
console = console_util.console


def resolve_import_target(target: str) -> Any:
    """
    中文:
      解析命令行传入的导入目标字符串，返回对应对象（函数/类/可调用等）。

    English:
      Resolve an import target string from CLI into the referenced object (function/class/callable).

    允许的导入路径格式 Allowed formats:
      1) module.submodule:attr
         - 使用冒号分隔模块与属性；attr 可继续包含 "." 以访问多级属性。
      2) module.submodule.attr
         - 不含冒号时，视为最后一个点号分隔模块与属性。

    相对路径规则 Relative path rules:
      - 不支持以 "." 开头的相对导入（例如 ".mymod:factory"）。
      - 传入的模块路径按照 Python 的导入系统进行解析，起始于当前工作目录的可导入包环境。
        换言之，相对路径应转换为可导入的包名（确保含有 __init__.py），并从运行 a2c-computer 的工作目录可被 sys.path 找到。

    例如 Examples:
      - "my_pkg.my_mod:build_computer"
      - "my_pkg.my_mod.MyComputerSubclass"
      - "pkg.sub.mod:factories.computer_factory"

    Raises:
      ValueError: 当字符串没有包含有效的模块与属性分隔时，或以相对导入开头时。
      ModuleNotFoundError/AttributeError: 导入失败时抛出。
    """
    module_name = None
    attr_path = None
    if ":" in target:
        module_name, _, attr_path = target.partition(":")
    else:
        # 用最后一个点号拆分模块与属性
        if "." not in target:
            raise ValueError(f"无效的导入目标: {target!r}，需要形如 'pkg.mod:attr' 或 'pkg.mod.attr'")
        module_name, _, attr_path = target.rpartition(".")

    if not module_name or not attr_path or module_name.startswith("."):
        raise ValueError(
            f"无效的导入目标: {target!r}，不支持相对导入，必须提供完整模块路径",
        )

    module = importlib.import_module(module_name)
    obj: Any = module
    for part in attr_path.split("."):
        obj = getattr(obj, part)
    return obj


def parse_kv_pairs(text: str | None) -> dict[str, Any] | None:
    """
    中文: 将形如 "k1:v1,k2:v2" 的字符串解析为 dict；容错处理空格。
    English: Parse a string like "k1:v1,k2:v2" into a dict; tolerant to spaces.

    Args:
        text: 原始输入字符串；None 或空字符串时返回 None。

    Returns:
        dict 或 None / dict or None
    """
    if text is None:
        return None
    s = text.strip()
    if s == "":
        return None
    # 优先尝试 JSON 反序列化：支持直接传入合法的 JSON 对象字符串
    # Try JSON deserialization first: support passing a valid JSON object string directly
    try:
        parsed = json.loads(s)
    except Exception:
        pass
    else:
        if isinstance(parsed, dict):
            return parsed
        # 合法 JSON 但不是对象（如数组/字符串/数字），给出明确错误
        raise ValueError('JSON 字符串必须是对象类型（例如 {"k":"v"}）')
    result: dict[str, Any] = {}
    for seg in s.split(","):
        seg = seg.strip()
        if seg == "":
            continue
        if ":" not in seg:
            raise ValueError(f"无效的键值对: {seg}，应为 key:value 形式")
        k, v = seg.split(":", 1)
        k = k.strip()
        v = v.strip()
        if not k:
            raise ValueError(f"无效的键名: '{seg}'")
        result[k] = v
    return result if result else None


def print_status(comp: Computer) -> None:
    """
    中文: 打印系统状态，包括 MCP 服务器状态和 SocketIO 连接状态。
    English: Print system status including MCP servers and SocketIO connection.
    """
    # 中文: 打印 SocketIO 连接状态 / English: Print SocketIO connection status
    client = comp.socketio_client
    if client:
        # 中文: 已连接，展示详细信息 / English: Connected, show details
        conn_table = Table(title="SocketIO 连接状态 / SocketIO Connection Status", show_header=True)
        conn_table.add_column("属性 / Property", style="cyan", no_wrap=True)
        conn_table.add_column("值 / Value", style="green")

        # 连接状态
        is_connected = client.connected
        conn_table.add_row(
            "连接状态 / Connected",
            "[green]✓ 已连接 / Connected[/green]" if is_connected else "[red]✗ 未连接 / Disconnected[/red]",
        )

        # 连接 URL
        # 中文: 从 EngineIO 层获取连接 URL / English: Get connection URL from EngineIO layer
        eio = getattr(client, "eio", None)
        if eio:
            base_url = getattr(eio, "base_url", "N/A")
        else:
            base_url = "N/A"
        conn_table.add_row("Server URL", str(base_url))

        # 客户端 ID (EngineIO SID)
        # 中文: 客户端总 ID，区别于各 Namespace 的 SID / English: Client ID (EngineIO level), different from namespace SIDs
        client_sid = getattr(eio, "sid", "N/A") if eio else "N/A"
        conn_table.add_row("Client ID", str(client_sid))

        # Office ID (房间)
        office_id = getattr(client, "office_id", None)
        conn_table.add_row(
            "Office ID (房间)",
            str(office_id) if office_id else "[dim]未加入 / Not joined[/dim]",
        )

        console.print(conn_table)
        console.print()  # 空行分隔

        # 中文: 打印所有已连接的 Namespace 信息 / English: Print all connected namespaces info
        if client.namespaces:
            ns_table = Table(title="Namespace 连接信息 / Namespace Connections", show_header=True)
            ns_table.add_column("Namespace", style="cyan", no_wrap=True)
            ns_table.add_column("SID", style="yellow")

            for namespace, sid in client.namespaces.items():
                ns_table.add_row(namespace, str(sid))

            console.print(ns_table)
            console.print()  # 空行分隔
        else:
            console.print("[dim]暂无 Namespace 连接 / No namespace connected[/dim]")
            console.print()
    else:
        # 中文: 未连接 / English: Not connected
        console.print("[yellow]SocketIO: 目前尚未连接 / Not connected yet[/yellow]")
        console.print()

    # 中文: 打印 MCP 服务器状态 / English: Print MCP servers status
    if not comp.mcp_manager:
        console.print("[yellow]MCP Manager 未初始化 / MCP Manager not initialized[/yellow]")
        return

    rows = comp.mcp_manager.get_server_status()
    table = Table(title="MCP 服务器状态 / MCP Servers Status")
    table.add_column("Name", style="cyan")
    table.add_column("Active", style="magenta")
    table.add_column("State", style="yellow")
    for name, active, state in rows:
        table.add_row(name, "[green]✓[/green]" if active else "[red]✗[/red]", state)
    console.print(table)


def print_tools(tools: list[SMCPTool]) -> None:
    """
    中文: 打印工具列表。
    English: Print tools list.
    """
    table = Table(title="工具列表 / Tools")
    table.add_column("Name")
    table.add_column("Description")
    table.add_column("Json Return")
    for t in tools:
        table.add_row(t.get("name", ""), (t.get("description") or "")[:80], "Yes" if t.get("return_schema") else "No")
    console.print(table)


def print_mcp_config(config: dict[str, Any]) -> None:
    """
    中文: 打印当前 MCP 配置（servers 与 inputs）。
    English: Print current MCP config (servers and inputs).
    """
    servers = config.get("servers") or {}
    inputs = config.get("inputs") or []
    console.print("[bold]Servers:[/bold]")
    s_table = Table()
    s_table.add_column("Name")
    s_table.add_column("Type")
    s_table.add_column("Disabled")
    for name, cfg in servers.items():
        s_table.add_row(name, cfg.get("type", ""), "Yes" if cfg.get("disabled") else "No")
    console.print(s_table)

    console.print("[bold]Inputs:[/bold]")
    i_table = Table()
    i_table.add_column("ID")
    i_table.add_column("Type")
    i_table.add_column("Description")
    for i in inputs:
        i_table.add_row(i.get("id", ""), i.get("type", ""), (i.get("description") or "")[:60])
    console.print(i_table)



"""
文件名: test_cli_run_with_config.py
作者: JQQ
创建日期: 2025/9/22
版权: 2023 JQQ. All rights reserved.
依赖: pytest, pexpect
描述:
  中文: 启动 CLI 时通过 --config/-c 传入配置文件，验证服务加载与工具可见。
  English: Pass --config at startup to load servers config and verify tools.
"""

from __future__ import annotations

import os
import re
import shutil
import signal
import sys
import time
from contextlib import contextmanager

import pytest

from tests.e2e.computer.utils import strip_ansi

pexpect = pytest.importorskip("pexpect", reason="e2e tests require pexpect; install with `pip install pexpect`.")


# ANSI-aware prompt matching and helper to strip ANSI sequences
# 与 conftest/test_cli_interactive 保持一致，确保在包含控制序列/光标移动的终端下也能稳定匹配提示符
ANSI = r"(?:\x1b\[[0-?]*[ -/]*[@-~])*"
PROMPT_RE = re.compile(ANSI + r"a2c>" + ANSI)


@contextmanager
def _spawn_cli_with_args(*extra_args: str):
    env = os.environ.copy()
    env.setdefault("PYTHONUNBUFFERED", "1")
    env.setdefault("PROMPT_TOOLKIT_NO_CPR", "1")
    env.setdefault("PROMPT_TOOLKIT_DISABLE_BRACKETED_PASTE", "1")
    env.setdefault("TERM", "dumb")
    console_script = shutil.which("a2c-computer")
    if console_script:
        args = [console_script, "--no-color", "run", *extra_args]
    else:
        args = [
            sys.executable,
            "-c",
            "from a2c_smcp_cc.cli.main import main; main()",
            "--no-color",
            "run",
            *extra_args,
        ]
    # 计算与设置工作目录 / Compute and set working directory
    # 默认将工作目录设置为项目根目录（本文件位于 tests/e2e/conftest.py，向上两级即为项目根）
    # By default, set cwd to project root (this file lives at tests/e2e/conftest.py; go up two levels)
    project_root = os.path.abspath(os.path.join(os.path.dirname(__file__), "..", "..", ".."))
    child = pexpect.spawn(args[0], args[1:], env=env, encoding="utf-8", timeout=25, cwd=project_root)
    try:
        child.setwinsize(24, 120)
    except Exception:
        pass
    try:
        yield child
    finally:
        if child.isalive():
            try:
                child.sendline("exit")
                child.expect([pexpect.EOF, "Bye"], timeout=5)
            except Exception:
                pass
        if child.isalive():
            try:
                child.kill(signal.SIGKILL)
            except Exception:
                pass


def _wait_prompt(child: pexpect.spawn, timeout: float = 15.0) -> None:
    child.expect(PROMPT_RE, timeout=timeout)


def _assert_tools(child: pexpect.spawn, name: str, retries: int = 10, delay: float = 1.0) -> None:
    for _ in range(retries):
        child.sendline("tools")
        _wait_prompt(child)
        out = strip_ansi((child.before or "").strip())
        if name in out:
            return
        time.sleep(delay)
    child.sendline("tools")
    _wait_prompt(child)
    out = strip_ansi((child.before or "").strip())
    assert name in out, f"tools 未包含 {name}. 输出:\n{out}"


def _assert_status(child: pexpect.spawn, server: str, retries: int = 8, delay: float = 0.8) -> None:
    for _ in range(retries):
        child.sendline("status")
        _wait_prompt(child)
        out = strip_ansi((child.before or "").strip())
        if server in out:
            return
        time.sleep(delay)
    child.sendline("status")
    _wait_prompt(child)
    out = strip_ansi((child.before or "").strip())
    assert server in out, f"status 未出现 {server}. 输出:\n{out}"


@pytest.mark.e2e
def test_run_with_config_param_loads_server() -> None:
    """
    启动参数包含 --config @tests/e2e/computer/configs/server_direct_execution.json，应能加载并启动服务：
    - 进入 a2c> 后检查 status 含 e2e-test
    - tools 中包含 hello
    若自动启动存在延迟，调用一次 start all 作为补偿
    """
    cfg_arg = "--config=@tests/e2e/computer/configs/server_direct_execution.json"
    with _spawn_cli_with_args(cfg_arg) as child:
        # 等横幅/提示符
        try:
            child.expect("Enter interactive mode, type 'help' for commands")
        except Exception:
            pass
        # 可能需要轻推以出现提示符
        for _ in range(5):
            try:
                _wait_prompt(child, timeout=5)
                break
            except pexpect.TIMEOUT:
                child.sendline("")
        else:
            _wait_prompt(child, timeout=10)

        # 若 auto-connect 未马上激活，补打一遍 start all
        child.sendline("start all")
        _wait_prompt(child)

        _assert_status(child, "e2e-test", retries=10, delay=0.8)
        _assert_tools(child, "hello", retries=12, delay=1.0)



"""
合并版 CLI 单测，包含基础与扩展用例
"""

from __future__ import annotations

import json
from collections.abc import Callable
from contextlib import contextmanager
from pathlib import Path
from typing import Any

import pytest

import a2c_smcp.computer.cli.main as cli_main
from a2c_smcp.computer.cli.main import _interactive_loop
from a2c_smcp.computer.computer import Computer


class DummyInteractive:
    called: bool = False
    last_comp: Any | None = None
    last_init_client: Any | None = None

    @classmethod
    async def coro(cls, comp: Any, init_client: Any | None = None) -> None:  # matches _interactive_loop signature
        cls.called = True
        cls.last_comp = comp
        cls.last_init_client = init_client


class FakeComputer:
    """A lightweight fake that matches Computer's init signature and async context manager."""

    def __init__(
        self,
        name: str,
        inputs: set[Any] | None = None,
        mcp_servers: set[Any] | None = None,
        auto_connect: bool = True,
        auto_reconnect: bool = True,
        confirm_callback: Callable[[str, str, str, dict], bool] | None = None,
        input_resolver: Any | None = None,
    ) -> None:
        self.init_args = {
            "inputs": inputs,
            "mcp_servers": mcp_servers,
            "auto_connect": auto_connect,
            "auto_reconnect": auto_reconnect,
            "confirm_callback": confirm_callback,
            "input_resolver": input_resolver,
        }

    async def __aenter__(self) -> FakeComputer:
        return self

    async def __aexit__(self, exc_type, exc, tb) -> None:  # noqa: ANN001
        return None


def test_run_impl_uses_default_computer_when_no_factory(monkeypatch: pytest.MonkeyPatch) -> None:
    # Patch Computer to our fake and _interactive_loop to a dummy coro
    monkeypatch.setattr(cli_main, "Computer", FakeComputer, raising=True)
    monkeypatch.setattr(cli_main, "_interactive_loop", DummyInteractive.coro, raising=True)

    # Call implementation with no factory and no side-effect options
    cli_main._run_impl(
        auto_connect=True,
        auto_reconnect=True,
        url=None,
        namespace=None,
        auth=None,
        headers=None,
        computer_factory=None,
        config=None,
        inputs=None,
    )

    assert DummyInteractive.called is True
    assert isinstance(DummyInteractive.last_comp, FakeComputer)
    assert DummyInteractive.last_comp.init_args["auto_connect"] is True
    assert DummyInteractive.last_comp.init_args["auto_reconnect"] is True


def test_run_impl_uses_resolved_factory(monkeypatch: pytest.MonkeyPatch) -> None:
    # Prepare a factory that returns our FakeComputer
    calls: dict[str, Any] = {"count": 0}

    def factory(**kwargs: Any) -> FakeComputer:
        calls["count"] += 1
        return FakeComputer(**kwargs)

    # Patch resolver to return our factory; patch interactive loop to avoid blocking
    monkeypatch.setattr(cli_main, "resolve_import_target", lambda s: factory, raising=True)
    monkeypatch.setattr(cli_main, "_interactive_loop", DummyInteractive.coro, raising=True)

    cli_main._run_impl(
        auto_connect=False,
        auto_reconnect=False,
        url=None,
        namespace=None,
        auth=None,
        headers=None,
        computer_factory="some.module:factory",
        config=None,
        inputs=None,
    )

    assert calls["count"] == 1
    assert isinstance(DummyInteractive.last_comp, FakeComputer)
    assert DummyInteractive.last_comp.init_args["auto_connect"] is False
    assert DummyInteractive.last_comp.init_args["auto_reconnect"] is False


def test_run_impl_factory_not_callable_fallback(monkeypatch: pytest.MonkeyPatch) -> None:
    # Make resolve_import_target return a non-callable
    monkeypatch.setattr(cli_main, "resolve_import_target", lambda s: object(), raising=True)
    # Patch Computer fallback to our FakeComputer
    monkeypatch.setattr(cli_main, "Computer", FakeComputer, raising=True)
    monkeypatch.setattr(cli_main, "_interactive_loop", DummyInteractive.coro, raising=True)

    cli_main._run_impl(
        auto_connect=True,
        auto_reconnect=True,
        url=None,
        namespace=None,
        auth=None,
        headers=None,
        computer_factory="x.y:bad",
        config=None,
        inputs=None,
    )

    assert isinstance(DummyInteractive.last_comp, FakeComputer)


def test_run_impl_resolve_error_fallback(monkeypatch: pytest.MonkeyPatch) -> None:
    def _raise(_: str) -> Any:
        raise ValueError("boom")

    monkeypatch.setattr(cli_main, "resolve_import_target", _raise, raising=True)
    monkeypatch.setattr(cli_main, "Computer", FakeComputer, raising=True)
    monkeypatch.setattr(cli_main, "_interactive_loop", DummyInteractive.coro, raising=True)

    cli_main._run_impl(
        auto_connect=True,
        auto_reconnect=True,
        url=None,
        namespace=None,
        auth=None,
        headers=None,
        computer_factory="x.y:z",
        config=None,
        inputs=None,
    )

    assert isinstance(DummyInteractive.last_comp, FakeComputer)


class FakePromptSession:
    """Feed scripted inputs to the interactive loop."""

    def __init__(self, commands: list[str]) -> None:
        self._commands = commands

    async def prompt_async(self, *_: str, **__: Any) -> str:  # noqa: D401
        if not self._commands:
            raise EOFError
        return self._commands.pop(0)


@contextmanager
def no_patch_stdout():
    """No-op context manager to replace patch_stdout() in tests."""
    yield


@pytest.mark.asyncio
async def test_interactive_help_and_exit(monkeypatch: pytest.MonkeyPatch) -> None:
    commands = [
        "help",
        "exit",
    ]
    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(commands))
    monkeypatch.setattr(cli_main, "patch_stdout", lambda raw: no_patch_stdout())

    comp = Computer(name="test_main_c", inputs=set(), mcp_servers=set(), auto_connect=False, auto_reconnect=False)
    await _interactive_loop(comp)


@pytest.mark.asyncio
async def test_server_add_exception_and_rm_with_client(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
    """覆盖 server add 的异常打印分支，以及 rm 时已连接触发 emit 分支。"""

    # server 配置文件
    server_file = tmp_path / "server.json"
    server_file.write_text(
        json.dumps(
            {
                "name": "s2",
                "type": "stdio",
                "disabled": True,
                "forbidden_tools": [],
                "tool_meta": {},
                "server_parameters": {
                    "command": "echo",
                    "args": [],
                    "env": None,
                    "cwd": None,
                    "encoding": "utf-8",
                    "encoding_error_handler": "strict",
                },
            },
        ),
        encoding="utf-8",
    )

    # 指令：先连接，再尝试 add 触发异常，再 rm 触发已连接 emit
    commands = [
        "socket connect http://localhost:9001",
        f"server add @{server_file}",
        "server rm s2",
        "exit",
    ]

    # 准备 comp 与补丁
    comp = Computer(name="test_main_c", inputs=set(), mcp_servers=set(), auto_connect=False, auto_reconnect=False)

    async def _raise_add(*args: Any, **kwargs: Any) -> None:  # noqa: ANN001
        raise RuntimeError("boom")

    monkeypatch.setattr(comp, "aadd_or_aupdate_server", _raise_add)
    monkeypatch.setattr(cli_main, "SMCPComputerClient", FakeSMCPClient)
    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(commands))
    monkeypatch.setattr(cli_main, "patch_stdout", lambda raw: no_patch_stdout())

    await _interactive_loop(comp)


@pytest.mark.asyncio
async def test_inputs_load_usage_and_success_with_client(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
    """覆盖 inputs load 的用法提示与成功路径（含 emit）。"""
    inputs_file = tmp_path / "inputs.json"
    inputs_file.write_text(
        json.dumps(
            [
                {"id": "J1", "type": "promptString", "description": "d", "default": "v"},
            ],
        ),
        encoding="utf-8",
    )

    commands = [
        "inputs load",  # 触发用法提示
        "socket connect http://localhost:9002",
        f"inputs load @{inputs_file}",  # 成功并触发 emit
        "exit",
    ]

    monkeypatch.setattr(cli_main, "SMCPComputerClient", FakeSMCPClient)
    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(commands))
    monkeypatch.setattr(cli_main, "patch_stdout", lambda raw: no_patch_stdout())

    comp = Computer(name="test_main_c", inputs=set(), mcp_servers=set(), auto_connect=False, auto_reconnect=False)
    await _interactive_loop(comp)


@pytest.mark.asyncio
async def test_socket_connect_guided_parse_error(monkeypatch: pytest.MonkeyPatch) -> None:
    """覆盖交互式 socket connect 的参数解析失败分支。"""
    commands = [
        "socket connect",
        "http://localhost:9003",
        "bad_auth_kv",  # 无效，触发 parse_kv_pairs 异常
        "exit",
    ]

    monkeypatch.setattr(cli_main, "SMCPComputerClient", FakeSMCPClient)
    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(commands))
    monkeypatch.setattr(cli_main, "patch_stdout", lambda raw: no_patch_stdout())

    comp = Computer(name="test_main_c", inputs=set(), mcp_servers=set(), auto_connect=False, auto_reconnect=False)
    await _interactive_loop(comp)


@pytest.mark.asyncio
async def test_inputs_value_print_json_fallback(monkeypatch: pytest.MonkeyPatch) -> None:
    """通过让 console.print_json 抛异常覆盖 repr 回退分支。"""
    import a2c_smcp.computer.cli.utils as cli_utils

    commands = [
        'inputs add {"id":"Z","type":"promptString","description":"d"}',
        'inputs value set Z {"x":1}',  # 设置为字典
        "inputs value get Z",  # 获取时让 print_json 抛错
        "exit",
    ]

    def _raise_print_json(*args: Any, **kwargs: Any) -> None:  # noqa: ANN001
        raise ValueError("no json")

    monkeypatch.setattr(cli_utils.console, "print_json", _raise_print_json, raising=True)
    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(commands))
    monkeypatch.setattr(cli_main, "patch_stdout", lambda raw: no_patch_stdout())

    comp = Computer(name="test_main_c", inputs=set(), mcp_servers=set(), auto_connect=False, auto_reconnect=False)
    await _interactive_loop(comp)


def test_run_impl_inputs_and_servers_single_object(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
    """覆盖 _run_impl 的 inputs/config 单对象路径。"""
    # 立即退出的交互
    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(["exit"]))
    monkeypatch.setattr(cli_main, "patch_stdout", lambda raw: no_patch_stdout())

    inputs_file = tmp_path / "i.json"
    inputs_file.write_text(
        json.dumps({"id": "SO", "type": "promptString", "description": "d", "default": "a"}),
        encoding="utf-8",
    )

    server_file = tmp_path / "s.json"
    server_file.write_text(
        json.dumps(
            {
                "name": "solo",
                "type": "stdio",
                "disabled": True,
                "forbidden_tools": [],
                "tool_meta": {},
                "server_parameters": {
                    "command": "echo",
                    "args": [],
                    "env": None,
                    "cwd": None,
                    "encoding": "utf-8",
                    "encoding_error_handler": "strict",
                },
            },
        ),
        encoding="utf-8",
    )

    cli_main._run_impl(
        auto_connect=False,
        auto_reconnect=False,
        url=None,
        namespace=cli_main.SMCP_NAMESPACE,
        auth=None,
        headers=None,
        computer_factory=None,
        config=str(server_file),  # 单对象
        inputs=str(inputs_file),  # 单对象
    )


@pytest.mark.asyncio
async def test_cover_remaining_branches(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
    """覆盖 interactive_impl.py 中剩余未命中的分支。"""
    # 为 inputs update @file 准备文件（列表）
    upd_file = tmp_path / "upd.json"
    upd_file.write_text(
        json.dumps(
            [
                {"id": "U1", "type": "promptString", "description": "d"},
                {"id": "U2", "type": "promptString", "description": "d2"},
            ],
        ),
        encoding="utf-8",
    )

    # 命令序列
    commands = [
        # 添加 server 后立刻 mcp，覆盖 servers 循环
        '{"cmd":"server add inline"}',  # 占位，下一行是真正的 add
        'server add {"name":"m1","type":"stdio","disabled":true,"forbidden_tools":[],"tool_meta":{},'
        '"server_parameters":{"command":"echo","args":[],"env":null,"cwd":null,"encoding":"utf-8","encoding_error_handler":"strict"}}',
        "mcp",
        # start/stop 时 manager 未初始化
        "start one",
        "stop one",
        # inputs add 用法
        "inputs add",
        # inputs update 用法 + @file 列表
        "inputs update",
        f"inputs update @{upd_file}",
        # inputs rm 用法 + rm 不存在
        "inputs rm",
        "inputs rm NOPE",
        # inputs get 用法
        "inputs get",
        # inputs value 顶层用法 + set 缺少参数 + set 不存在 id + get 不存在值
        "inputs value",
        "inputs value set",
        "inputs value set NOPE 1",
        "inputs value get NOPE",
        # inputs value 未知子命令
        "inputs value what",
        # socket connect 引导但 URL 为空，触发 URL required
        "socket connect",
        "",
        # socket join 带参数但尚未连接
        "socket join o1 c1",
        # socket leave 在未连接
        "socket leave",
        "exit",
    ]

    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(commands))
    monkeypatch.setattr(cli_main, "patch_stdout", lambda raw: no_patch_stdout())

    comp = Computer(name="test_main_c", inputs=set(), mcp_servers=set(), auto_connect=False, auto_reconnect=False)
    await _interactive_loop(comp)


@pytest.mark.asyncio
async def test_interactive_misc_and_file_paths(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
    """覆盖更多 interactive_impl 分支：
    - 空输入跳过
    - tools/mcp 打印
    - server add 使用 @file + 随后 rm
    - inputs add 使用 @file（数组）与 update 使用单对象
    - inputs value 边界：缺少参数、指定 id 清理、JSON 载荷
    - socket 再次 connect 走 already-connected 分支
    - socket join/leave 的未连接/未加入分支
    - 未知子命令（server/socket/notify）与 render 内联 JSON
    - start/stop 单个名称（manager 初始化后触发路径）
    """

    # 预备文件：server 与 inputs
    server_file = tmp_path / "server.json"
    server_file.write_text(
        json.dumps(
            {
                "name": "s1",
                "type": "stdio",
                "disabled": True,  # 避免真实启动
                "forbidden_tools": [],
                "tool_meta": {},
                "server_parameters": {
                    "command": "echo",
                    "args": [],
                    "env": None,
                    "cwd": None,
                    "encoding": "utf-8",
                    "encoding_error_handler": "strict",
                },
            },
        ),
        encoding="utf-8",
    )

    inputs_file = tmp_path / "inputs.json"
    inputs_file.write_text(
        json.dumps(
            [
                {"id": "I1", "type": "promptString", "description": "d1", "default": "x"},
                {"id": "I2", "type": "pickString", "description": "d2", "options": ["a", "b"], "default": "a"},
            ],
        ),
        encoding="utf-8",
    )

    # 指令脚本
    commands = [
        "",  # 空输入
        "tools",
        "mcp",
        f"server add @{server_file}",
        "server rm s1",
        f"inputs add @{inputs_file}",  # 数组 add
        'inputs update {"id":"I1","type":"promptString","description":"d1u","default":"y"}',  # 单对象 update
        "inputs value get",  # 缺失 id
        "inputs value rm",  # 缺失 id
        'inputs value set I1 {"k":1}',  # JSON 载荷
        "inputs value clear I1",  # 指定 id 清理
        "socket connect http://localhost:9000",  # 连接一次
        "socket connect http://localhost:9000",  # 已连接分支
        "socket join",  # 缺少参数
        "socket leave",  # 未加入房间
        "server unknownsub",
        "socket unknown",
        "notify unknown",
        'render {"a":1}',  # 内联 JSON 渲染
        # 初始化 manager 后测试 start/stop 单个名称分支
        "exit",
    ]

    # 打补丁：Session/patch_stdout/SMCP 客户端与 tools 列表
    class LocalFakeClient(FakeSMCPClient):
        pass

    # 我们需要在交互开始前让 comp.manager 初始化，以便稍后可以测试 start/stop 单个名称
    # 这里分两段会话：第一段跑上述命令到 exit，然后第二段在 manager 初始化后再跑 start/stop name

    monkeypatch.setattr(cli_main, "SMCPComputerClient", LocalFakeClient)
    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(commands))
    monkeypatch.setattr(cli_main, "patch_stdout", lambda raw: no_patch_stdout())

    comp = Computer(name="test_main_c", inputs=set(), mcp_servers=set(), auto_connect=False, auto_reconnect=False)

    # stub 工具列表
    async def _fake_tools() -> list[dict[str, Any]]:
        return [{"name": "t1", "description": "d", "return_schema": {}}]

    monkeypatch.setattr(comp, "aget_available_tools", _fake_tools)

    await _interactive_loop(comp)

    # 第二段：初始化 manager 后测试 start/stop <name> 分支（即使失败也能走异常打印分支）
    await comp.boot_up()
    commands2 = [
        "start xxx",
        "stop xxx",
        "exit",
    ]
    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(commands2))
    await _interactive_loop(comp)


def test_root_no_color_triggers_console_switch(monkeypatch: pytest.MonkeyPatch) -> None:
    """覆盖 _root 的 no_color 分支，并确保调用 _run_impl。"""
    called: dict[str, Any] = {"ok": False}

    def _stub_run_impl(**kwargs: Any) -> None:  # noqa: ANN003
        called["ok"] = True

    class Ctx:
        invoked_subcommand = None

    monkeypatch.setattr(cli_main, "_run_impl", _stub_run_impl, raising=True)

    # 验证不会抛异常，且 _run_impl 被调用
    cli_main._root(Ctx(), no_color=True)  # 其它参数用默认值
    assert called["ok"] is True


def test_run_impl_loads_inputs_and_servers_from_files(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
    """覆盖 _run_impl 的 inputs/config 文件加载成功路径。"""
    # 提供立即退出的交互
    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(["exit"]))
    monkeypatch.setattr(cli_main, "patch_stdout", lambda raw: no_patch_stdout())

    inputs_file = tmp_path / "inputs.json"
    inputs_file.write_text(
        json.dumps(
            [
                {"id": "VA", "type": "promptString", "description": "d", "default": "1"},
                {"id": "VB", "type": "pickString", "description": "d", "options": ["x", "y"], "default": "x"},
            ],
        ),
        encoding="utf-8",
    )

    server_file = tmp_path / "servers.json"
    server_file.write_text(
        json.dumps(
            [
                {
                    "name": "s1",
                    "type": "stdio",
                    "disabled": True,
                    "forbidden_tools": [],
                    "tool_meta": {},
                    "server_parameters": {
                        "command": "echo",
                        "args": [],
                        "env": None,
                        "cwd": None,
                        "encoding": "utf-8",
                        "encoding_error_handler": "strict",
                    },
                },
            ],
        ),
        encoding="utf-8",
    )

    # 运行：不提供 url，避免网络；仅加载文件
    cli_main._run_impl(
        auto_connect=False,
        auto_reconnect=False,
        url=None,
        namespace=cli_main.SMCP_NAMESPACE,
        auth=None,
        headers=None,
        computer_factory=None,
        config=str(server_file),
        inputs=str(inputs_file),
    )


def test_run_impl_cli_params_parse_error(monkeypatch: pytest.MonkeyPatch) -> None:
    """覆盖 _run_impl 在解析 auth/headers 失败时的异常分支。"""
    # 立即退出
    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(["exit"]))
    monkeypatch.setattr(cli_main, "patch_stdout", lambda raw: no_patch_stdout())
    # 使用假的 Socket 客户端避免真实连接
    monkeypatch.setattr(cli_main, "SMCPComputerClient", FakeSMCPClient)

    # 传入无效的 kv 字符串（缺少冒号），触发 parse_kv_pairs 抛错，从而走 except 分支
    cli_main._run_impl(
        auto_connect=False,
        auto_reconnect=False,
        url="http://localhost:7777",
        namespace=cli_main.SMCP_NAMESPACE,
        auth="invalid",  # 无效
        headers="also_invalid",  # 无效
        computer_factory=None,
        config=None,
        inputs=None,
    )


@pytest.mark.asyncio
async def test_inputs_cli_crud_commands(monkeypatch: pytest.MonkeyPatch) -> None:
    """覆盖 inputs 子命令：add/update/rm/get/list，并在连接状态下触发配置更新通知。"""
    monkeypatch.setattr(cli_main, "SMCPComputerClient", FakeSMCPClient)

    # 使用 socket connect 建立连接，随后执行 inputs 的 CRUD 命令
    commands = [
        "socket connect http://localhost:7000",
        # add 单条
        'inputs add {"id":"USER","type":"promptString","description":"d","default":"a"}',
        # get + list
        "inputs get USER",
        "inputs list",
        # update 批量（数组）
        'inputs update [{"id":"USER","type":"promptString","description":"d2","default":"b"},'
        ' {"id":"REG","type":"pickString","description":"r","options":["us","eu"],"default":"us"}]',
        "inputs list",
        # rm
        "inputs rm USER",
        "inputs list",
        "exit",
    ]

    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(commands))
    monkeypatch.setattr(cli_main, "patch_stdout", lambda raw: no_patch_stdout())

    comp = Computer(name="test_main_c", inputs=set(), mcp_servers=set(), auto_connect=False, auto_reconnect=False)
    await _interactive_loop(comp)

    last: FakeSMCPClient = FakeSMCPClient.last  # type: ignore[assignment]
    # 至少在 add/update/rm 期间触发了多次更新通知
    assert last.updated >= 3


@pytest.mark.asyncio
async def test_socket_connect_guided_inputs_parsing(monkeypatch: pytest.MonkeyPatch) -> None:
    """
    验证在未提供 URL 的情况下，交互式引导输入 URL/Auth/Headers，并正确解析传给 connect(auth=..., headers=...).
    """
    monkeypatch.setattr(cli_main, "SMCPComputerClient", FakeSMCPClient)

    # 触发引导式：先输入命令，再依次回应 URL、Auth、Headers，然后退出
    commands = [
        "socket connect",
        "http://localhost:8000",
        "token:abc123",
        "app:demo,ver:1.0",
        "exit",
    ]

    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(commands))
    monkeypatch.setattr(cli_main, "patch_stdout", lambda raw: no_patch_stdout())

    comp = Computer(name="test_main_c", inputs=set(), mcp_servers=set(), auto_connect=False, auto_reconnect=False)
    await _interactive_loop(comp)

    # 断言 FakeSMCPClient 收到了期望的参数
    last: FakeSMCPClient = FakeSMCPClient.last  # type: ignore[assignment]
    assert last.connected is True
    assert last.connect_args is not None
    assert last.connect_args["url"] == "http://localhost:8000"
    assert last.connect_args["auth"] == {"token": "abc123"}
    assert last.connect_args["headers"] == {"app": "demo", "ver": "1.0"}


def test_run_with_cli_url_auth_headers(monkeypatch: pytest.MonkeyPatch) -> None:
    """
    验证通过 run(url=..., auth=..., headers=...) 启动时，会自动连接并传入解析后的参数，随后进入交互并退出。
    """
    monkeypatch.setattr(cli_main, "SMCPComputerClient", FakeSMCPClient)

    # 进入交互后立即退出
    commands = [
        "exit",
    ]
    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(commands))
    monkeypatch.setattr(cli_main, "patch_stdout", lambda raw: no_patch_stdout())

    # 调用同步的 run()，其内部使用 asyncio.run() 执行
    cli_main.run(
        auto_connect=False,
        auto_reconnect=False,
        url="http://service:1234",
        namespace=cli_main.SMCP_NAMESPACE,
        auth="token:abc",
        headers="h1:v1,h2:v2",
    )

    last: FakeSMCPClient = FakeSMCPClient.last  # type: ignore[assignment]
    assert last.connected is True
    assert last.connect_args == {
        "url": "http://service:1234",
        "auth": {"token": "abc"},
        "headers": {"h1": "v1", "h2": "v2"},
        "namespaces": [cli_main.SMCP_NAMESPACE],
    }


@pytest.mark.asyncio
async def test_server_add_and_status_without_auto_connect(monkeypatch: pytest.MonkeyPatch) -> None:
    # Minimal stdio server config (disabled=true to avoid start operations later)
    stdio_cfg = {
        "name": "test-stdio",
        "type": "stdio",
        "disabled": True,
        "forbidden_tools": [],
        "tool_meta": {},
        "server_parameters": {
            "command": "echo",
            "args": [],
            "env": None,
            "cwd": None,
            "encoding": "utf-8",
            "encoding_error_handler": "strict",
        },
    }

    commands = [
        f"server add {stdio_cfg}",
        "mcp",
        "status",
        "exit",
    ]

    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(commands))
    monkeypatch.setattr(cli_main, "patch_stdout", lambda raw: no_patch_stdout())

    comp = Computer(name="test_main_c", inputs=set(), mcp_servers=set(), auto_connect=False, auto_reconnect=False)
    await _interactive_loop(comp)


@pytest.mark.asyncio
async def test_unknown_and_status_manager_uninitialized(monkeypatch: pytest.MonkeyPatch) -> None:
    commands = [
        "unknown",
        "status",
        "exit",
    ]
    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(commands))
    monkeypatch.setattr(cli_main, "patch_stdout", lambda raw: no_patch_stdout())

    comp = Computer(name="test_main_c", inputs=set(), mcp_servers=set(), auto_connect=False, auto_reconnect=False)
    await _interactive_loop(comp)


@pytest.mark.asyncio
async def test_server_rm_without_name_and_add_invalid_json(monkeypatch: pytest.MonkeyPatch) -> None:
    commands = [
        "server rm",
        "server add {invalid}",
        "exit",
    ]
    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(commands))
    monkeypatch.setattr(cli_main, "patch_stdout", lambda raw: no_patch_stdout())

    comp = Computer(name="test_main_c", inputs=set(), mcp_servers=set(), auto_connect=False, auto_reconnect=False)
    await _interactive_loop(comp)


@pytest.mark.asyncio
async def test_start_stop_all_with_manager_initialized(monkeypatch: pytest.MonkeyPatch) -> None:
    comp = Computer(name="test_main_c", inputs=set(), mcp_servers=set(), auto_connect=False, auto_reconnect=False)
    await comp.boot_up()

    commands = [
        "start all",
        "stop all",
        "exit",
    ]
    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(commands))
    monkeypatch.setattr(cli_main, "patch_stdout", lambda raw: no_patch_stdout())

    await _interactive_loop(comp)


@pytest.mark.asyncio
async def test_inputs_load_and_render(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
    inputs_file = tmp_path / "inputs.json"
    inputs_file.write_text(
        json.dumps(
            [
                {"id": "VAR1", "type": "promptString", "description": "v", "default": "abc"},
                {"id": "CHOICE", "type": "pickString", "description": "d", "options": ["x", "y"], "default": "x"},
            ],
        ),
        encoding="utf-8",
    )

    any_file = tmp_path / "any.json"
    any_file.write_text(json.dumps({"k": "${input:VAR1}", "c": "${input:CHOICE}"}), encoding="utf-8")

    commands = [
        f"inputs load @{inputs_file}",
        f"render @{any_file}",
        "exit",
    ]

    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(commands))
    monkeypatch.setattr(cli_main, "patch_stdout", lambda raw: no_patch_stdout())

    comp = Computer(name="test_main_c", inputs=set(), mcp_servers=set(), auto_connect=False, auto_reconnect=False)
    await _interactive_loop(comp)


class FakeSMCPClient:
    def __init__(self, *args: Any, **kwargs: Any) -> None:  # noqa: D401
        self.connected = False
        self.office_id: str | None = None
        self.joined_args: tuple[str, str] | None = None
        self.updated = 0
        # 记录最后一个实例，便于断言
        FakeSMCPClient.last = self  # type: ignore[attr-defined]
        self.connect_args: dict[str, Any] | None = None

    async def connect(
        self,
        url: str,
        auth: dict[str, Any] | None = None,
        headers: dict[str, Any] | None = None,
        namespaces: list[str] | None = None,
    ) -> None:
        self.connected = True
        args: dict[str, Any] = {"url": url, "auth": auth, "headers": headers}
        if namespaces is not None:
            args["namespaces"] = namespaces
        self.connect_args = args

    async def join_office(self, office_id: str, computer_name: str) -> None:
        assert self.connected
        self.office_id = office_id
        self.joined_args = (office_id, computer_name)

    async def leave_office(self, office_id: str) -> None:
        assert self.connected
        self.office_id = None

    async def emit_update_config(self) -> None:
        self.updated += 1


@pytest.mark.asyncio
async def test_socket_and_notify_branches(monkeypatch: pytest.MonkeyPatch) -> None:
    monkeypatch.setattr(cli_main, "SMCPComputerClient", FakeSMCPClient)

    commands = [
        "notify update",
        "socket connect http://localhost:7000",
        "socket join office-1 compA",
        "notify update",
        "socket leave",
        "exit",
    ]

    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(commands))
    monkeypatch.setattr(cli_main, "patch_stdout", lambda raw: no_patch_stdout())

    comp = Computer(name="test_main_c", inputs=set(), mcp_servers=set(), auto_connect=False, auto_reconnect=False)
    await _interactive_loop(comp)



"""
中文: 最小可运行的 MCP Stdio 服务器，用于集成测试。
英文: Minimal runnable MCP Stdio server for integration tests.

文件名: minimal_stdio_server.py / filename: minimal_stdio_server.py
作者: JQQ / author: JQQ
创建日期: 2025/9/28 / created at: 2025/9/28
最后修改日期: 2025/9/28 / last modified at: 2025/9/28
版权: 2023 JQQ. All rights reserved. / copyright: 2023 JQQ. All rights reserved.
依赖: anyio, mcp / dependencies: anyio, mcp
描述: 启动一个最小的基于 stdio 的 MCP 服务器，仅用于握手与初始化测试。/ description: Start a minimal stdio-based MCP server
    for handshake/initialize tests.
"""

import anyio
from mcp.server.lowlevel.server import Server
from mcp.server.stdio import stdio_server


async def run() -> None:
    """
    中文: 启动服务器，使用默认能力，不注册任何工具/资源，仅支持初始化。
    英文: Run the server with default capabilities, no tools/resources, only initialization support.
    """
    server = Server(name="itest-server", version="0.0.1", instructions="itest")

    async with stdio_server() as (read_stream, write_stream):
        init_opts = server.create_initialization_options()
        await server.run(read_stream, write_stream, init_opts)


if __name__ == "__main__":
    anyio.run(run)



# -*- coding: utf-8 -*-
# filename: proxy.py
# @Time    : 2025/8/18 17:08
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm


class AwaitableOnly:
    """This wraps a coroutine will call it on await."""

    def __init__(self, coro):
        object.__setattr__(self, "_coro", coro)

    def __repr__(self):
        return f'<AwaitableOnly "{self._coro.__qualname__}">'

    def __await__(self):
        return self._coro().__await__()

    __slots__ = ["_coro"]


"""Below taken from: https://github.com/GrahamDumpleton/wrapt/blob/master/src/wrapt/wrappers.py

Copyright (c) 2013-2019, Graham Dumpleton
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
  list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation
  and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
"""
import operator


class _ObjectProxyMethods:
    # We use properties to override the values of __module__ and
    # __doc__. If we add these in ObjectProxy, the derived class
    # __dict__ will still be setup to have string variants of these
    # attributes and the rules of descriptors means that they appear to
    # take precedence over the properties in the base class. To avoid
    # that, we copy the properties into the derived class type itself
    # via a meta class. In that way the properties will always take
    # precedence.

    @property
    def __module__(self):
        return self.__wrapped__.__module__

    @__module__.setter
    def __module__(self, value):
        self.__wrapped__.__module__ = value

    @property
    def __doc__(self):
        return self.__wrapped__.__doc__

    @__doc__.setter
    def __doc__(self, value):
        self.__wrapped__.__doc__ = value

    # We similar use a property for __dict__. We need __dict__ to be
    # explicit to ensure that vars() works as expected.

    @property
    def __dict__(self):
        return self.__wrapped__.__dict__

    # Need to also propagate the special __weakref__ attribute for case
    # where decorating classes which will define this. If do not define
    # it and use a function like inspect.getmembers() on a decorator
    # class it will fail. This can't be in the derived classes.

    @property
    def __weakref__(self):
        return self.__wrapped__.__weakref__


class _ObjectProxyMetaType(type):
    def __new__(cls, name, bases, dictionary):
        # Copy our special properties into the class so that they
        # always take precedence over attributes of the same name added
        # during construction of a derived class. This is to save
        # duplicating the implementation for them in all derived classes.

        dictionary.update(vars(_ObjectProxyMethods))

        return type.__new__(cls, name, bases, dictionary)


class ObjectProxy(metaclass=_ObjectProxyMetaType):
    __slots__ = "__wrapped__"

    def __init__(self, wrapped):
        object.__setattr__(self, "__wrapped__", wrapped)

        # Python 3.2+ has the __qualname__ attribute, but it does not
        # allow it to be overridden using a property and it must instead
        # be an actual string object instead.

        try:
            object.__setattr__(self, "__qualname__", wrapped.__qualname__)
        except AttributeError:
            pass

    @property
    def __name__(self):
        return self.__wrapped__.__name__

    @__name__.setter
    def __name__(self, value):
        self.__wrapped__.__name__ = value

    @property
    def __class__(self):
        return self.__wrapped__.__class__

    @__class__.setter
    def __class__(self, value):
        self.__wrapped__.__class__ = value

    @property
    def __annotations__(self):
        return self.__wrapped__.__annotations__

    @__annotations__.setter
    def __annotations__(self, value):
        self.__wrapped__.__annotations__ = value

    def __dir__(self):
        return dir(self.__wrapped__)

    def __str__(self):
        return str(self.__wrapped__)

    def __bytes__(self):
        return bytes(self.__wrapped__)

    def __repr__(self):
        return f"<{type(self).__name__} at 0x{id(self):x} for {type(self.__wrapped__).__name__} at 0x{id(self.__wrapped__):x}>"

    def __reversed__(self):
        return reversed(self.__wrapped__)

    def __round__(self):
        return round(self.__wrapped__)

    def __lt__(self, other):
        return self.__wrapped__ < other

    def __le__(self, other):
        return self.__wrapped__ <= other

    def __eq__(self, other):
        return self.__wrapped__ == other

    def __ne__(self, other):
        return self.__wrapped__ != other

    def __gt__(self, other):
        return self.__wrapped__ > other

    def __ge__(self, other):
        return self.__wrapped__ >= other

    def __hash__(self):
        return hash(self.__wrapped__)

    def __bool__(self):
        return bool(self.__wrapped__)

    def __setattr__(self, name, value):
        if name.startswith("_self_"):
            object.__setattr__(self, name, value)

        elif name == "__wrapped__":
            object.__setattr__(self, name, value)
            try:
                object.__delattr__(self, "__qualname__")
            except AttributeError:
                pass
            try:
                object.__setattr__(self, "__qualname__", value.__qualname__)
            except AttributeError:
                pass

        elif name == "__qualname__":
            setattr(self.__wrapped__, name, value)
            object.__setattr__(self, name, value)

        elif hasattr(type(self), name):
            object.__setattr__(self, name, value)

        else:
            setattr(self.__wrapped__, name, value)

    def __getattr__(self, name):
        # If we are being to lookup '__wrapped__' then the
        # '__init__()' method cannot have been called.

        if name == "__wrapped__":
            raise ValueError("wrapper has not been initialised")

        return getattr(self.__wrapped__, name)

    def __delattr__(self, name):
        if name.startswith("_self_"):
            object.__delattr__(self, name)

        elif name == "__wrapped__":
            raise TypeError("__wrapped__ must be an object")

        elif name == "__qualname__":
            object.__delattr__(self, name)
            delattr(self.__wrapped__, name)

        elif hasattr(type(self), name):
            object.__delattr__(self, name)

        else:
            delattr(self.__wrapped__, name)

    def __add__(self, other):
        return self.__wrapped__ + other

    def __sub__(self, other):
        return self.__wrapped__ - other

    def __mul__(self, other):
        return self.__wrapped__ * other

    def __truediv__(self, other):
        return operator.truediv(self.__wrapped__, other)

    def __floordiv__(self, other):
        return self.__wrapped__ // other

    def __mod__(self, other):
        return self.__wrapped__ % other

    def __divmod__(self, other):
        return divmod(self.__wrapped__, other)

    def __pow__(self, other, *args):
        return pow(self.__wrapped__, other, *args)

    def __lshift__(self, other):
        return self.__wrapped__ << other

    def __rshift__(self, other):
        return self.__wrapped__ >> other

    def __and__(self, other):
        return self.__wrapped__ & other

    def __xor__(self, other):
        return self.__wrapped__ ^ other

    def __or__(self, other):
        return self.__wrapped__ | other

    def __radd__(self, other):
        return other + self.__wrapped__

    def __rsub__(self, other):
        return other - self.__wrapped__

    def __rmul__(self, other):
        return other * self.__wrapped__

    def __rtruediv__(self, other):
        return operator.truediv(other, self.__wrapped__)

    def __rfloordiv__(self, other):
        return other // self.__wrapped__

    def __rmod__(self, other):
        return other % self.__wrapped__

    def __rdivmod__(self, other):
        return divmod(other, self.__wrapped__)

    def __rpow__(self, other, *args):
        return pow(other, self.__wrapped__, *args)

    def __rlshift__(self, other):
        return other << self.__wrapped__

    def __rrshift__(self, other):
        return other >> self.__wrapped__

    def __rand__(self, other):
        return other & self.__wrapped__

    def __rxor__(self, other):
        return other ^ self.__wrapped__

    def __ror__(self, other):
        return other | self.__wrapped__

    def __iadd__(self, other):
        self.__wrapped__ += other
        return self

    def __isub__(self, other):
        self.__wrapped__ -= other
        return self

    def __imul__(self, other):
        self.__wrapped__ *= other
        return self

    def __itruediv__(self, other):
        self.__wrapped__ = operator.itruediv(self.__wrapped__, other)
        return self

    def __ifloordiv__(self, other):
        self.__wrapped__ //= other
        return self

    def __imod__(self, other):
        self.__wrapped__ %= other
        return self

    def __ipow__(self, other):
        self.__wrapped__ **= other
        return self

    def __ilshift__(self, other):
        self.__wrapped__ <<= other
        return self

    def __irshift__(self, other):
        self.__wrapped__ >>= other
        return self

    def __iand__(self, other):
        self.__wrapped__ &= other
        return self

    def __ixor__(self, other):
        self.__wrapped__ ^= other
        return self

    def __ior__(self, other):
        self.__wrapped__ |= other
        return self

    def __neg__(self):
        return -self.__wrapped__

    def __pos__(self):
        return +self.__wrapped__

    def __abs__(self):
        return abs(self.__wrapped__)

    def __invert__(self):
        return ~self.__wrapped__

    def __int__(self):
        return int(self.__wrapped__)

    def __float__(self):
        return float(self.__wrapped__)

    def __complex__(self):
        return complex(self.__wrapped__)

    def __oct__(self):
        return oct(self.__wrapped__)

    def __hex__(self):
        return hex(self.__wrapped__)

    def __index__(self):
        return operator.index(self.__wrapped__)

    def __len__(self):
        return len(self.__wrapped__)

    def __contains__(self, value):
        return value in self.__wrapped__

    def __getitem__(self, key):
        return self.__wrapped__[key]

    def __setitem__(self, key, value):
        self.__wrapped__[key] = value

    def __delitem__(self, key):
        del self.__wrapped__[key]

    def __getslice__(self, i, j):
        return self.__wrapped__[i:j]

    def __setslice__(self, i, j, value):
        self.__wrapped__[i:j] = value

    def __delslice__(self, i, j):
        del self.__wrapped__[i:j]

    def __enter__(self):
        return self.__wrapped__.__enter__()

    def __exit__(self, *args, **kwargs):
        return self.__wrapped__.__exit__(*args, **kwargs)

    def __iter__(self):
        return iter(self.__wrapped__)

    def __copy__(self):
        raise NotImplementedError("object proxy must define __copy__()")

    def __deepcopy__(self, memo):
        raise NotImplementedError("object proxy must define __deepcopy__()")

    def __reduce__(self):
        raise NotImplementedError("object proxy must define __reduce_ex__()")

    def __reduce_ex__(self, protocol):
        raise NotImplementedError("object proxy must define __reduce_ex__()")

    def __call__(self, *args, **kwargs):
        return self.__wrapped__(*args, **kwargs)


class AwaitableProxy(ObjectProxy):
    def __await__(self):
        async def get_wrapped():
            return self.__wrapped__

        return get_wrapped().__await__()

    async def __aenter__(self):
        return await self.__wrapped__.__aenter__()

    async def __aexit__(self, *args, **kwargs):
        return await self.__wrapped__.__aexit__(*args, **kwargs)

    async def __aiter__(self):
        return await self.__wrapped__.__aiter__()

    async def __anext__(self):
        return await self.__wrapped__.__anext__()



# -*- coding: utf-8 -*-
# filename: test_interactive_history.py
# @Time    : 2025/10/02 14:02
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
"""
文件名: test_interactive_history.py
作者: JQQ
创建日期: 2025/10/02
最后修改日期: 2025/10/02
版权: 2023 JQQ. All rights reserved.
依赖: pytest
描述:
  中文: 集成测试层面对 `history` 命令做一次基本打通验证：先伪造一次调用记录，再读取 history。
  English: Integration-level sanity check for `history`: pre-populate one record then read it back.
"""

from __future__ import annotations

from contextlib import contextmanager

import pytest

import a2c_smcp.computer.cli.main as cli_main
from a2c_smcp.computer.cli.main import _interactive_loop
from a2c_smcp.computer.computer import Computer


class FakePromptSession:
    def __init__(self, commands: list[str]) -> None:
        self._commands = commands

    async def prompt_async(self, *_: str, **__: object) -> str:  # noqa: D401
        if not self._commands:
            raise EOFError
        return self._commands.pop(0)


async def _append_history(comp: Computer, **kwargs):  # noqa: D401
    await comp._append_tool_history(kwargs)  # 内部API用于测试打桩，生产不会直接使用


@pytest.mark.asyncio
async def test_history_basic_integration(monkeypatch: pytest.MonkeyPatch) -> None:
    # 预置命令
    commands = ["history", "exit"]
    monkeypatch.setattr(cli_main, "PromptSession", lambda: FakePromptSession(commands))
    # 使用空操作的上下文管理器替代 patch_stdout
    # Use a no-op context manager to replace patch_stdout

    @contextmanager
    def _noop_ctx(*args, **kwargs):
        yield

    monkeypatch.setattr(cli_main, "patch_stdout", lambda raw: _noop_ctx())

    comp = Computer(name="test", inputs=set(), mcp_servers=set(), auto_connect=False, auto_reconnect=False)

    # 伪造一条历史记录（直接调用专用于测试的方法）
    await _append_history(
        comp,
        timestamp="2025-10-02T05:00:00Z",
        req_id="R",
        server="S",
        tool="T",
        parameters={"k": 1},
        timeout=None,
        success=True,
        error=None,
    )

    await _interactive_loop(comp)

    assert True



# filename: model.py
# @Time    : 2025/8/17 16:53
# @Author  : JQQ
# @Email   : jiaqia@qknode.com
# @Software: PyCharm
from typing import TYPE_CHECKING, ClassVar, Literal, Protocol, TypeAlias, runtime_checkable

from mcp import StdioServerParameters, Tool
from mcp.client.session_group import SseServerParameters, StreamableHttpParameters
from mcp.types import CallToolResult, ReadResourceResult, Resource
from pydantic import BaseModel, ConfigDict, Field, field_validator
from vrl_python import VRLRuntime

from a2c_smcp.types import SERVER_NAME, TOOL_NAME

if TYPE_CHECKING:
    from a2c_smcp.computer.mcp_clients.base_client import STATES

A2C_TOOL_META: str = "a2c_tool_meta"
# VRL转换后的结果存储Key。用于在CallToolResult.meta中存储VRL处理后的数据。
# Key for storing VRL-transformed result in CallToolResult.meta.
A2C_VRL_TRANSFORMED: str = "a2c_vrl_transformed"


class ToolMeta(BaseModel):
    auto_apply: bool | None = Field(default=None, title="是否自动使用", description="如果设置为False，则调用工具前会触发回调，请求用户批准")
    alias: str | None = Field(
        default=None,
        title="工具别名",
        description="如果不同MCP Server中存在同名工具，允许通过此别名修改，从而解决名称冲突",
    )
    tags: list[str] | None = Field(default=None, title="工具标签", description="用于对工具进行分类")
    # 不同MCP工具返回值并不统一，虽然其满足MCP标准的返回格式，但具体的原始内容命名仍然无法避免出现不一致的情况。通过object_mapper可以方便
    # 前端对其进行转换，以使用标准组件渲染解析。
    ret_object_mapper: dict | None = Field(
        default=None,
        title="字段转换映射",
        description="允许定义一个映射表完成MCPTool工具返回结构映射到自定义结构",
    )

    model_config: ClassVar[ConfigDict] = ConfigDict(extra="allow", arbitrary_types_allowed=False)


class BaseMCPServerConfig(BaseModel):
    """MCP服务器配置基类"""

    name: SERVER_NAME  # MCP Server的名称
    disabled: bool = Field(default=False, title="是否禁用", description="是否禁用MCP Server")
    forbidden_tools: list[TOOL_NAME] = Field(
        default_factory=list,
        title="禁用工具列表",
        description="禁用的工具列表，因为一个mcp可能有非常多工具，有些工具用户需要禁用。",
    )
    tool_meta: dict[TOOL_NAME, ToolMeta] = Field(default_factory=dict, title="工具元数据", description="工具元数据，用于描述工具的基本信息")
    # 默认工具元数据（可选）。当某个具体工具未在 tool_meta 中提供专门配置时，使用该默认配置。
    # Default tool metadata (optional). Used when a specific tool has no explicit entry in tool_meta.
    default_tool_meta: ToolMeta | None = Field(default=None)
    # VRL脚本（可选）。用于对工具返回值进行动态转换和格式化。如果配置了VRL脚本，在初始化时会进行语法检查。
    # VRL script (optional). Used to dynamically transform and format tool return values. Syntax check on initialization.
    vrl: str | None = Field(
        default=None,
        title="VRL脚本",
        description="用于对工具返回值进行动态转换和格式化的VRL脚本。配置后会在初始化时进行语法检查。",
    )

    model_config: ClassVar[ConfigDict] = ConfigDict(extra="forbid", arbitrary_types_allowed=False, frozen=True)
    """配置字段在初始化完成后不允许修改"""

    @field_validator("vrl")
    @classmethod
    def validate_vrl_syntax(cls, v: str | None) -> str | None:
        """
        验证VRL脚本语法。如果配置了VRL脚本但语法错误，则抛出异常。
        Validate VRL script syntax. Raises exception if VRL is configured but has syntax errors.

        Args:
            v (str | None): VRL脚本内容 / VRL script content

        Returns:
            str | None: 验证通过的VRL脚本 / Validated VRL script

        Raises:
            ValueError: VRL语法错误 / VRL syntax error
        """
        if v is None or v.strip() == "":
            return v

        # 使用VRL Runtime进行语法检查
        # Use VRL Runtime for syntax check
        diagnostic = VRLRuntime.check_syntax(v)
        if diagnostic is not None:
            # 语法错误，抛出异常
            # Syntax error, raise exception
            error_msg = f"VRL语法错误 / VRL syntax error:\n{diagnostic.formatted_message}"
            raise ValueError(error_msg)

        return v

    def __hash__(self) -> int:
        """对于MCP Server配置，以name作为唯一标识凭证，如果name相同则表示完全相同"""
        return hash(self.name)


class StdioServerConfig(BaseMCPServerConfig):
    type: Literal["stdio"] = "stdio"
    server_parameters: StdioServerParameters = Field(title="MCP Server启动参数", description="引用自MCP Python SDK官方配置")


class SseServerConfig(BaseMCPServerConfig):
    type: Literal["sse"] = "sse"
    server_parameters: SseServerParameters = Field(title="MCP SSE Server连接参数", description="引用自MCP Python SDK 官方配置")


class StreamableHttpServerConfig(BaseMCPServerConfig):
    type: Literal["streamable"] = "streamable"
    server_parameters: StreamableHttpParameters = Field(title="MCP HTTP Server连接参数", description="引用自MCP Python SDK 官方配置")


MCPServerConfig: TypeAlias = StdioServerConfig | SseServerConfig | StreamableHttpServerConfig


class MCPServerInputBase(BaseModel):
    """MCP服务器输入项配置基类"""

    id: str
    """Input的唯一标准，即使跨类型，也不可重复"""
    description: str

    model_config: ClassVar[ConfigDict] = ConfigDict(extra="forbid", arbitrary_types_allowed=False, frozen=True)
    """配置字段在初始化完成后不允许修改"""

    def __hash__(self) -> int:
        """以 id 作为唯一哈希，确保在 set 中按 id 去重"""
        return hash(self.id)

    def __eq__(self, other: object) -> bool:
        """按 id 判断相等性，确保不同内容但相同 id 的输入在集合中视为同一元素"""
        if not isinstance(other, MCPServerInputBase):
            return False
        return self.id == other.id


class MCPServerPromptStringInput(MCPServerInputBase):
    """字符串输入类型，参考：https://code.visualstudio.com/docs/reference/variables-reference#_input-variables"""

    type: Literal["promptString"] = Field(default="promptString")
    default: str | None = Field(default=None)
    password: bool | None = Field(default=None)


class MCPServerPickStringInput(MCPServerInputBase):
    """选择输入类型，参考：https://code.visualstudio.com/docs/reference/variables-reference#_input-variables"""

    type: Literal["pickString"] = Field(default="pickString")
    options: list[str] = Field(default_factory=list)
    default: str | None = Field(default=None)


class MCPServerCommandInput(MCPServerInputBase):
    """命令输入类型，参考：https://code.visualstudio.com/docs/reference/variables-reference#_input-variables"""

    type: Literal["command"] = Field(default="command")
    command: str = Field(title="")
    args: dict[str, str] | None = Field(default=None)


MCPServerInput = MCPServerPromptStringInput | MCPServerPickStringInput | MCPServerCommandInput


@runtime_checkable
class MCPClientProtocol(Protocol):
    state: "STATES"

    async def aconnect(self) -> None:
        """连接MCP Server"""
        ...

    async def adisconnect(self) -> None:
        """断开连接"""
        ...

    async def list_tools(self) -> list[Tool]:
        """获取可用工具列表"""
        return []

    async def call_tool(self, tool_name: str, params: dict) -> CallToolResult:
        """运行指定工具"""
        pass

    async def list_windows(self) -> list[Resource]:
        """列出当前MCP服务可用的窗口资源列表 / List window resources of the MCP server"""
        ...

    async def get_window_detail(self, resource: Resource | str) -> ReadResourceResult:
        """获取当前Window的详细内容"""
        ...



# -*- coding: utf-8 -*-
# filename: __init__.py.py
# @Time    : 2025/8/18 17:08
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
from .base import async_property

__all__ = ["async_property"]



"""
文件名: test_cli_mcp_config_start.py
作者: JQQ
创建日期: 2025/9/22
版权: 2023 JQQ. All rights reserved.
依赖: pytest, pexpect
描述:
  中文: 使用固定的配置文件 tests/e2e/computer/configs/server_direct_execution.json 来添加并启动 stdio MCP Server，然后校验 status 与 tools。
  English: Use fixed config file to add/start stdio MCP Server and assert status/tools.
"""

from __future__ import annotations

import pytest

from tests.e2e.computer.utils import expect_prompt_stable, strip_ansi

pexpect = pytest.importorskip("pexpect", reason="e2e tests require pexpect; install with `pip install pexpect`.")


def _assert_contains_tools(child: pexpect.spawn, tool_name: str, retries: int = 10, delay: float = 1.0) -> None:
    """检查工具列表中是否包含指定的工具名称 / Check if tools list contains the specified tool name"""
    # 获取详细输出用于调试 / Final attempt with detailed output for debugging
    child.sendline("tools")
    out = expect_prompt_stable(child, quiet=0.4, max_wait=12.0)
    clean_out = strip_ansi(out)
    assert tool_name in clean_out, f"tools 未包含 {tool_name}. 尝试次数: {retries}. 清理后输出:\n{clean_out}\n原始输出:\n{out}"


def _assert_status_has(child: pexpect.spawn, server_name: str, retries: int = 10, delay: float = 1.0) -> None:
    """检查服务器状态中是否包含指定的服务器名称 / Check if server status contains the specified server name"""
    # 获取详细输出用于调试 / Final attempt with detailed output for debugging
    child.sendline("status")
    out = expect_prompt_stable(child, quiet=0.4, max_wait=12.0)
    clean_out = strip_ansi(out)
    assert server_name in clean_out, f"status 未出现 {server_name}. 尝试次数: {retries}. 清理后输出:\n{clean_out}\n原始输出:\n{out}"


@pytest.mark.e2e
def test_start_via_known_config_file(cli_proc: pexpect.spawn) -> None:
    """
    使用固定的配置文件路径添加并启动 direct_execution 服务器，然后验证状态与工具：
    - server add @tests/e2e/computer/configs/server_direct_execution.json
    - start all
    - status 包含 e2e-test
    - tools 包含 hello
    """
    child = cli_proc

    # 添加服务器配置 / Add server configuration
    child.sendline("server add @tests/e2e/computer/configs/server_direct_execution.json")
    # 等待稳定提示符，确保 add 的输出完全结束 / wait for stable prompt to ensure add output finished
    expect_prompt_stable(child, quiet=0.5, max_wait=15.0)

    # 验证服务器状态和工具可用性 / Verify server status and tool availability
    _assert_status_has(child, "e2e-test", retries=12, delay=1.0)
    _assert_contains_tools(child, "hello", retries=12, delay=1.0)



# -*- coding: utf-8 -*-
# filename: test_computer_desktop_and_edges.py
# @Time    : 2025/10/05 13:43
# @Author  : A2C-SMCP
# @Email   : qa@a2c-smcp.local
# @Software: PyTest
"""
中文：覆盖 `a2c_smcp/computer/computer.py` 缺失分支，包括：
- boot_up 渲染异常兜底（163-166）
- _on_manager_change 的 ToolListChanged/ResourceListChanged/ResourceUpdated 多分支（193-200, 205-233, 243-252）
- _acollect_window_uris（262-266）
- _arender_and_validate_server 异常分支（327-330）
- get_input/remove_input 边界返回（410, 434）

English: Cover missing branches in computer.py.
"""

from types import SimpleNamespace
from typing import Any

import pytest

from a2c_smcp.computer.computer import Computer
from a2c_smcp.computer.mcp_clients.model import MCPServerConfig, StdioServerConfig, StdioServerParameters
from a2c_smcp.utils.window_uri import is_window_uri


class _DummyClient:
    def __init__(self) -> None:
        self.update_called = 0
        self.refresh_called = 0

    async def emit_update_tool_list(self) -> None:
        self.update_called += 1

    async def emit_refresh_desktop(self) -> None:
        self.refresh_called += 1


@pytest.mark.asyncio
async def test_on_manager_change_tool_list_changed_triggers_emit(monkeypatch: pytest.MonkeyPatch) -> None:
    comp = Computer(name="test", auto_connect=False, auto_reconnect=False)
    # 绑定伪客户端
    client = _DummyClient()
    comp.socketio_client = client  # weakref setter

    # 直接设置类型名不方便，使用 mcp.types 实例化更复杂；这里依赖 isinstance 仅检查类名不可靠。
    # 因 computer.py 使用 'from mcp.types import ToolListChangedNotification' 并直接 isinstance，
    # 我们改为导入真实类型来创建对象。
    from mcp.types import ToolListChangedNotification  # type: ignore

    real_msg = SimpleNamespace(root=ToolListChangedNotification())

    await comp._on_manager_change(real_msg)  # type: ignore[arg-type]
    assert client.update_called == 1


@pytest.mark.asyncio
async def test_on_manager_change_resource_list_changed_paths(monkeypatch: pytest.MonkeyPatch) -> None:
    comp = Computer(name="test", auto_connect=False, auto_reconnect=False)
    # 绑定伪客户端
    client = _DummyClient()
    comp.socketio_client = client

    from mcp.types import ResourceListChangedNotification  # type: ignore

    # 1) _acollect_window_uris 抛异常 -> 日志后返回，不触发刷新
    async def raise_collect() -> set[str]:
        raise RuntimeError("collect error")

    monkeypatch.setattr(comp, "_acollect_window_uris", raise_collect)
    await comp._on_manager_change(SimpleNamespace(root=ResourceListChangedNotification()))  # type: ignore
    assert client.refresh_called == 0

    # 2) 集合发生变化 -> 触发刷新
    async def changed_collect() -> set[str]:
        return {"window://a"}

    monkeypatch.setattr(comp, "_acollect_window_uris", changed_collect)
    comp._windows_cache = set()  # 原为空
    await comp._on_manager_change(SimpleNamespace(root=ResourceListChangedNotification()))  # type: ignore
    assert client.refresh_called == 1

    # 3) 集合未变化 -> 不触发刷新
    comp._windows_cache = {"window://a"}

    async def same_collect() -> set[str]:
        return {"window://a"}

    monkeypatch.setattr(comp, "_acollect_window_uris", same_collect)
    await comp._on_manager_change(SimpleNamespace(root=ResourceListChangedNotification()))  # type: ignore
    assert client.refresh_called == 1  # 未增加


@pytest.mark.asyncio
async def test_on_manager_change_resource_updated_window_and_nonwindow() -> None:
    comp = Computer(name="test", auto_connect=False, auto_reconnect=False)
    client = _DummyClient()
    comp.socketio_client = client

    from mcp.types import ResourceUpdatedNotification, ResourceUpdatedNotificationParams  # type: ignore

    # 非 window 资源 -> 不触发
    non_window = ResourceUpdatedNotification(params=ResourceUpdatedNotificationParams(uri="file://x"))
    await comp._on_manager_change(SimpleNamespace(root=non_window))  # type: ignore
    assert client.refresh_called == 0

    # window 资源 -> 触发
    window = ResourceUpdatedNotification(params=ResourceUpdatedNotificationParams(uri="window://y"))
    assert is_window_uri("window://y")
    await comp._on_manager_change(SimpleNamespace(root=window))  # type: ignore
    assert client.refresh_called == 1


@pytest.mark.asyncio
async def test_acollect_window_uris_filters_and_none_manager(monkeypatch: pytest.MonkeyPatch) -> None:
    comp = Computer(name="test", auto_connect=False, auto_reconnect=False)

    # manager 未初始化 -> 返回空
    assert await comp._acollect_window_uris() == set()

    # 注入假的 manager 和资源
    class _Res:  # noqa: B903
        def __init__(self, uri: str) -> None:
            self.uri = uri

    class _Mgr:
        async def list_windows(self, window_uri: str | None = None):  # noqa: D401
            return [("s1", _Res("window://ok")), ("s2", _Res("file://ignored"))]

    comp.mcp_manager = _Mgr()  # type: ignore
    uris = await comp._acollect_window_uris()
    assert uris == {"window://ok"}


@pytest.mark.asyncio
async def test_boot_up_render_error_path(monkeypatch: pytest.MonkeyPatch) -> None:
    # 构造一个初始 server 配置
    cfg = StdioServerConfig(name="s", server_parameters=StdioServerParameters(command="/bin/echo"))
    # 注入自定义 Computer，覆写 _config_render.arender 抛出异常
    comp = Computer(name="test", mcp_servers={cfg})

    class _CR:
        async def arender(self, *_: Any, **__: Any) -> dict:
            raise RuntimeError("render fail")

    comp._config_render = _CR()  # type: ignore

    # 替换 MCPServerManager 以避免真实调用
    class _Mgr:
        async def ainitialize(self, servers: list[MCPServerConfig]) -> None:  # noqa: D401
            # 应接受到原始 cfg，因为渲染失败走保底
            assert isinstance(servers[0], MCPServerConfig)

    monkeypatch.setattr("a2c_smcp.computer.computer.MCPServerManager", lambda *a, **k: _Mgr())

    await comp.boot_up()


@pytest.mark.asyncio
async def test_arender_and_validate_server_exception_branch(monkeypatch: pytest.MonkeyPatch) -> None:
    comp = Computer(name="test")

    class _CR:
        async def arender(self, *_: Any, **__: Any) -> dict:
            raise RuntimeError("render fail 2")

    comp._config_render = _CR()  # type: ignore

    # 传入字典，触发渲染异常 -> except 分支记录日志后抛出
    with pytest.raises(RuntimeError):
        await comp._arender_and_validate_server({"type": "stdio", "name": "n", "server_parameters": {"command": "x"}})


def test_get_input_and_remove_input_boundaries() -> None:
    comp = Computer(name="test")
    # get_input 空字符串 -> None
    assert comp.get_input("") is None
    # remove_input 空字符串 -> False
    assert comp.remove_input("") is False



# -*- coding: utf-8 -*-
# filename: test_desktop_command.py
# @Time    : 2025/10/02 20:16
# @Author  : JQQ
# @Email   : jqq1716@gmail.com
# @Software: PyCharm
"""
中文: CLI 新增的 desktop 命令集成测试。
英文: Integration tests for the newly added CLI 'desktop' command.
"""

from __future__ import annotations

import json
import sys
from contextlib import contextmanager
from pathlib import Path
from typing import Any

import pytest
from mcp import StdioServerParameters

import a2c_smcp.computer.cli.main as cli_main
from a2c_smcp.computer.cli.main import _interactive_loop
from a2c_smcp.computer.computer import Computer


class FakePromptSession:
    def __init__(self, commands: list[str]) -> None:
        self._commands = commands

    async def prompt_async(self, *_: str, **__: Any) -> str:  # noqa: D401
        if not self._commands:
            raise EOFError
        return self._commands.pop(0)


@contextmanager
def no_patch_stdout():
    yield


@pytest.mark.asyncio
async def test_cli_desktop_with_subscribe_resources_server() -> None:
    """
    中文: 使用 resources_subscribe_stdio_server 启动后，执行 desktop 命令应输出非空列表。
    英文: After starting resources_subscribe_stdio_server, 'desktop' should output a non-empty list.
    """
    server_py = Path(__file__).resolve().parents[2] / "computer" / "mcp_servers" / "resources_subscribe_stdio_server.py"
    assert server_py.exists()

    params = StdioServerParameters(command=sys.executable, args=[str(server_py)])

    server_cfg = {
        "name": "it-res-sub",
        "type": "stdio",
        "disabled": False,
        "forbidden_tools": [],
        "tool_meta": {},
        "server_parameters": json.loads(params.model_dump_json()),
    }

    # 执行流程：添加 -> 启动 -> desktop -> 退出
    commands = [
        f"server add {json.dumps(server_cfg)}",
        "start it-res-sub",
        "desktop",
        "exit",
    ]

    # Patch interactive IO
    cli_main.PromptSession = lambda: FakePromptSession(commands)  # type: ignore
    cli_main.patch_stdout = lambda raw: no_patch_stdout()  # type: ignore

    comp = Computer(name="test", inputs=set(), mcp_servers=set(), auto_connect=False, auto_reconnect=False)

    # 为断言 desktop 的输出可调用 comp.get_desktop 再次核验
    await _interactive_loop(comp)
    desktops = await comp.get_desktop()
    assert isinstance(desktops, list) and len(desktops) >= 1


@pytest.mark.asyncio
async def test_cli_help_contains_desktop() -> None:
    """
    中文: help 列表应包含 desktop 命令说明。
    英文: 'help' listing should contain the 'desktop' command description.
    """
    commands = [
        "help",
        "exit",
    ]

    cli_main.PromptSession = lambda: FakePromptSession(commands)  # type: ignore
    cli_main.patch_stdout = lambda raw: no_patch_stdout()  # type: ignore

    comp = Computer(name="test", inputs=set(), mcp_servers=set(), auto_connect=False, auto_reconnect=False)
    await _interactive_loop(comp)
